#include <stdio.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#define __USE_GNU
#include <string.h>
#include <unistd.h>
#include "fftv.h"
#include "tv-win.h"

static GtkWidget *opt_format, *menu_format;
static GtkWidget *opt_pict_size, *menu_pict_size;
static GtkWidget *codec_window;
static GtkWidget *entry_file;
static GtkWidget *spinner_framerate, *spinner_bitrate;
static GtkWidget *arrow_codec;
static GtkWidget *separator_codec_hbox;

typedef struct {
  char *format;
  char **codecArr;
  int codecArrN;
  GtkWidget *menu;
  int active_codec_idx;
} FORMAT_CODEC;

static FORMAT_CODEC *formatArr;
static int formatArrN;

static int active_format_idx, active_pict_size_idx;

static GtkWidget *button_label_codec;
extern char *skip_spc(char *s), *to_spc(char *s);
char *pict_sizeArr[]={"720x480", "640x480", "320x240", "160x120"};
int pict_sizeArrN=4;

void add_format_codec(char *format_codec)
{
  char **codecArr=NULL;
  int codecArrN=0;

  char *format=skip_spc(format_codec);
  char *p=to_spc(format);
  int formatlen = p - format;

  while (*p) {
    p = skip_spc(p);
    char *q = to_spc(p);
    int len = q - p;
    codecArr=realloc(codecArr,sizeof(char *) * (codecArrN + 1));
    codecArr[codecArrN++]=strndup(p, len);
    p = q;
  }

  formatArr=realloc(formatArr, sizeof(FORMAT_CODEC) * (formatArrN + 1));
  FORMAT_CODEC *pformat = &formatArr[formatArrN];
  pformat->codecArr = codecArr;
  pformat->codecArrN = codecArrN;
  pformat->format = strndup(format, formatlen);
  pformat->menu = NULL;
  pformat->active_codec_idx = 0;
  formatArrN++;
}

static void hide_codec_arrow()
{
  gtk_widget_hide(arrow_codec);
  gtk_widget_hide(separator_codec_hbox);
}

void close_codec_ui_win()
{
  int i;

  for(i=0; i<formatArrN;i++) {
    FORMAT_CODEC *pformat = &formatArr[i];

    int j;

    for(j=0; j<pformat->codecArrN; j++)
      free(pformat->codecArr[j]);

    free(pformat->codecArr);
    free(pformat->format);

//    gtk_widget_destroy(pformat->menu);
  }

  free(formatArr); formatArr=NULL; formatArrN=0;

  if (codec_window)
    gtk_widget_destroy(codec_window);

  codec_window = NULL;
}

void set_format_codec_active_idx(char *format, char *codec)
{
  int i;

  for(i=0; i < formatArrN; i++)
    if (!strcmp(formatArr[i].format, format))
      break;

  if (i==formatArrN)
    return;

  active_format_idx = i;
  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_format), active_format_idx);

  if (!formatArr[i].codecArr)
    codec = NULL;

  if (!codec) {
    if (button_label_codec)
      gtk_label_set_text(GTK_LABEL(button_label_codec), "--");
    hide_codec_arrow();
    return;
  }

  FORMAT_CODEC *pformat = &formatArr[i];

  for(i=0; i < pformat->codecArrN; i++)
    if (!strcmp(pformat->codecArr[i], codec)) {
      pformat->active_codec_idx = i;
      break;
    }

  if (!pformat->codecArr) {
    hide_codec_arrow();
  }

  if (button_label_codec) {
    gtk_label_set_text(GTK_LABEL(button_label_codec), codec);
  }
}

void cb_update_menu_select(GtkWidget  *item,  GtkWidget *opt)
{
    /* Set the update policy for both scale widgets */

   active_format_idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));

   GtkWidget *menu = formatArr[active_format_idx].menu;

   if (!menu) {
     gtk_label_set_text(GTK_LABEL(button_label_codec), "--");
     gtk_widget_hide(arrow_codec);
     gtk_widget_hide(separator_codec_hbox);
     return;
   } else {
     gtk_widget_show(arrow_codec);
     gtk_widget_show(separator_codec_hbox);
   }

   FORMAT_CODEC *pformat = &formatArr[active_format_idx];

   gtk_label_set_text(GTK_LABEL(button_label_codec),
      pformat->codecArr[pformat->active_codec_idx]);
}


void cb_update_menu_select_codec(GtkWidget  *item,  gpointer data)
{
    /* Set the update policy for both scale widgets */
   int idx=GPOINTER_TO_INT(data);
   FORMAT_CODEC *pformat = &formatArr[active_format_idx];
   char *codec = pformat->codecArr[idx];

   printf("%d %s\n",idx, codec);

   pformat->active_codec_idx = idx;
   gtk_label_set_text(GTK_LABEL(button_label_codec), codec);
}

void cb_update_menu_select_pict_size(GtkWidget  *item,  GtkWidget *opt)
{
    /* Set the update policy for both scale widgets */

   active_pict_size_idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));
   dbg("select pict %s\n",  pict_sizeArr[active_pict_size_idx]);
}

static gint
my_popup_handler (GtkWidget *widget, GdkEvent *event)
{
  GtkWidget *menu;
  GdkEventButton *event_button;

  menu = formatArr[active_format_idx].menu;

  if (!menu)
    return TRUE;

  g_return_val_if_fail (GTK_IS_MENU (menu), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  if (event->type == GDK_BUTTON_PRESS) {
      event_button = (GdkEventButton *) event;
      gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
                      event_button->button, event_button->time);
      return TRUE;
  }

  return FALSE;
}

void set_arg_val(char *argname, const char *argval);
void rm_arg(char *argname);
void save_current_channel_name();
void set_arg_val_int(char *argname, int val);

static void cb_ok()
{
  const char *text_file = gtk_entry_get_text(GTK_ENTRY(entry_file));

  FORMAT_CODEC *pformat = &formatArr[active_format_idx];
  char *codec = pformat->codecArr ?
                  pformat->codecArr[pformat->active_codec_idx] : NULL;
  char file_name[256];


  if (codec)
    set_arg_val("-vcodec", codec);
  else
    rm_arg("-vcodec");

  gint bitrate = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinner_bitrate));
  gint framerate = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinner_framerate));

  if (fftv_mode & FFTV_MODE_TV) {
    set_arg_val_int("-b", bitrate);
    set_arg_val_int("-r", framerate);
    set_arg_val("-s", pict_sizeArr[active_pict_size_idx]);
  }
  else
    set_arg_val_int("-ab", bitrate);

  sprintf(file_name, "%s.%s",text_file, pformat->format);
  set_arg_val(file_name, NULL);

  close_codec_ui_win();

  close_vfd();

  record_status = RECORD_STOP;

  save_current_channel_name();

  if (TV_WIN_DISP_MODE_GRAB) {
    create_x_shm_image();
  }

  gtk_main_quit();
  printf("ok\n");
}


static void cb_cancel()
{
  close_codec_ui_win();

  printf("cancel\n");
}

void get_fftv_states_dir(char *fftv_states_dir);

void init_arg()
{
  char fftv_states_dir[256];
  char *fname_mode_codecs;

  get_fftv_states_dir(fftv_states_dir);

  if (fftv_mode & FFTV_MODE_TV) {
    fname_mode_codecs = "codecs-tv";
  } else {
    fname_mode_codecs = "codecs-radio";
  }

  char fname[256];
  sprintf(fname, "%s/%s", FFTV_CONF, fname_mode_codecs);

  FILE *fp;
  if ((fp=fopen(fname,"r"))==NULL)
    p_err("cannot open %s\n",fname);

  while (!feof(fp)) {
    char buf[256];

    buf[0]=0;
    fgets(buf,sizeof(buf),fp);

    if (!buf[0])
      continue;

    if (buf[strlen(buf)-1]=='\n')
      buf[strlen(buf)-1]=0;

    if (!buf[0])
      continue;

    add_format_codec(buf);
  }

  fclose(fp);
}

extern char *orig_output_filename;
char *get_arg_val(char *argname);

static void set_active_pict_size_idx()
{
  char *pict_size = get_arg_val("-s");


  int i;
  for(i=0; i< pict_sizeArrN; i++) {
    if (!strcmp(pict_sizeArr[i],pict_size)) {
      active_pict_size_idx = i;
      break;
    }
  }

  dbg("pict_size: %s  %s\n", pict_size, pict_sizeArr[active_pict_size_idx]);
}


void create_codec_window()
{
    GtkWidget *hbox_file, *hbox_format, *hbox_codec, *hbox_bitrate, *hbox_framerate;
    GtkWidget *hbox_pict_size;
    GtkWidget *label_file, *label_format, *label_codec, *label_bitrate, *label_framerate;

    if (codec_window) {
      gtk_window_present(GTK_WINDOW(codec_window));
      return;
    }

    init_arg();

    char file_name[256], *file_format;

    strcpy(file_name, orig_output_filename);

//    dbg("orig_output_filename: %s\n", orig_output_filename);

    if (strchr(file_name, '.')) {
      char *p;

      for(p=file_name+strlen(file_name)-1; *p!='.'; p--);
      *p=0;
      file_format=p+1;
    } else
      file_format = "avi";

    codec_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_container_set_border_width (GTK_CONTAINER (codec_window), 10);

    g_signal_connect (G_OBJECT (codec_window), "delete_event",
                        G_CALLBACK (cb_cancel), NULL);

    gtk_window_set_title (GTK_WINDOW (codec_window), "File format/codec");

    opt_format = gtk_option_menu_new ();
    menu_format = gtk_menu_new ();

    GtkWidget *vbox = gtk_vbox_new (FALSE, 5);

    hbox_file = gtk_hbox_new (FALSE, 0);

    label_file = gtk_label_new("File name");
    gtk_box_pack_start (GTK_BOX (hbox_file), label_file, FALSE, TRUE, 0);

    entry_file = gtk_entry_new ();
#if 0
    g_signal_connect (G_OBJECT (entry), "activate",
		      G_CALLBACK (enter_callback),
		      (gpointer) entry);
#endif
    gtk_entry_set_max_length (GTK_ENTRY (entry_file), 128);
    gtk_entry_set_width_chars(GTK_ENTRY (entry_file), 12);
    gtk_entry_set_text (GTK_ENTRY (entry_file), file_name);
//    gtk_widget_set_size_request(entry, 100,24);
    gtk_box_pack_start (GTK_BOX (hbox_file), entry_file, FALSE, TRUE, 10);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_file, TRUE, TRUE, 0);

    GtkWidget *separator_file = gtk_hseparator_new ();

    gtk_box_pack_start (GTK_BOX (vbox), separator_file, TRUE, TRUE, 3);

    hbox_format = gtk_hbox_new (FALSE, 0);

    label_format = gtk_label_new("File format");
    gtk_box_pack_start (GTK_BOX (hbox_format), label_format, FALSE, TRUE, 0);

    int i;
    for(i=0; i < formatArrN; i++) {
      FORMAT_CODEC *pformat = &formatArr[i];

      GtkWidget *item = gtk_menu_item_new_with_label (pformat->format);
      g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK(cb_update_menu_select), opt_format);
      gtk_menu_shell_append (GTK_MENU_SHELL (menu_format), item);

      if (!pformat->codecArrN)
        continue;

      GtkWidget *menu = pformat->menu = gtk_menu_new ();

      int j;
      for(j=0;j< pformat->codecArrN;j++) {
        GtkWidget *item = gtk_menu_item_new_with_label (pformat->codecArr[j]);

        g_signal_connect (G_OBJECT (item), "activate",
                          G_CALLBACK (cb_update_menu_select_codec), GINT_TO_POINTER(j));

        gtk_widget_show(item);

        gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
      }
    }

    gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_format), menu_format);

    gtk_box_pack_start (GTK_BOX (hbox_format), opt_format, FALSE, TRUE, 10);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_format, TRUE, TRUE, 0);


    GtkWidget *separator_format = gtk_hseparator_new ();

    gtk_box_pack_start (GTK_BOX (vbox), separator_format, TRUE, TRUE, 3);

    hbox_codec = gtk_hbox_new (FALSE, 0);

    label_codec = gtk_label_new("Codec");
    gtk_box_pack_start (GTK_BOX (hbox_codec), label_codec, FALSE, TRUE, 0);

    GtkWidget *button_codec_hbox = gtk_hbox_new (FALSE, 0);

    GtkWidget *button_codec = gtk_button_new();
    button_label_codec = gtk_label_new("-");
    gtk_box_pack_start(GTK_BOX (button_codec_hbox), button_label_codec, FALSE, FALSE,0);
    separator_codec_hbox = gtk_vseparator_new ();
    gtk_box_pack_start(GTK_BOX (button_codec_hbox), separator_codec_hbox, FALSE, FALSE,2);
    arrow_codec = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
    gtk_box_pack_start(GTK_BOX (button_codec_hbox), arrow_codec, FALSE, FALSE,0);

    g_signal_connect_swapped (GTK_OBJECT (button_codec), "button_press_event",
          G_CALLBACK (my_popup_handler), NULL);

    gtk_container_add (GTK_CONTAINER (button_codec), button_codec_hbox);

    gtk_box_pack_start (GTK_BOX (hbox_codec), button_codec, FALSE, TRUE, 10);
    gtk_box_pack_start (GTK_BOX (vbox), hbox_codec, TRUE, TRUE, 0);

    GtkWidget *separator_codec = gtk_hseparator_new ();

    gtk_box_pack_start (GTK_BOX (vbox), separator_codec, TRUE, TRUE, 3);

    hbox_bitrate = gtk_hbox_new (FALSE, 0);

    label_bitrate = gtk_label_new("Bitrate");
    gtk_box_pack_start (GTK_BOX (hbox_bitrate), label_bitrate, FALSE, TRUE, 0);

    char *bitrate_str = get_arg_val((fftv_mode & FFTV_MODE_TV) ? "-b":"-ab");

    double step = (fftv_mode & FFTV_MODE_TV) ? 200:2;

    GtkAdjustment *adj_bitrate = (GtkAdjustment *) gtk_adjustment_new (
       atof(bitrate_str), 10.0, 4000.0, step, step, step);
    spinner_bitrate = gtk_spin_button_new (adj_bitrate, step, 0);
    gtk_widget_set_size_request (spinner_bitrate, 60, -1);
    gtk_box_pack_start (GTK_BOX (hbox_bitrate), spinner_bitrate, FALSE, FALSE, 10);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_bitrate, TRUE, TRUE, 0);

    GtkWidget *separator_bitrate = gtk_hseparator_new ();

    gtk_box_pack_start (GTK_BOX (vbox), separator_bitrate, TRUE, TRUE, 3);

    if (fftv_mode & FFTV_MODE_TV) {
      hbox_framerate = gtk_hbox_new (FALSE, 0);
      label_framerate = gtk_label_new("Frames/sec");
      gtk_box_pack_start (GTK_BOX (hbox_framerate), label_framerate, FALSE, FALSE, 0);

      char *frame_rate_str = get_arg_val("-r");

      GtkAdjustment *adj_framerate = (GtkAdjustment *) gtk_adjustment_new (
         atof(frame_rate_str), 1, 30, 1, 1, 5);
      spinner_framerate = gtk_spin_button_new (adj_framerate, 1, 0);
      gtk_widget_set_size_request (spinner_framerate, 45, -1);
      gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (spinner_framerate), TRUE);
      gtk_box_pack_start (GTK_BOX (hbox_framerate), spinner_framerate, FALSE, TRUE, 10);

      gtk_box_pack_start (GTK_BOX (vbox), hbox_framerate, TRUE, TRUE, 0);

      GtkWidget *separator_framerate = gtk_hseparator_new ();
      gtk_box_pack_start (GTK_BOX (vbox), separator_framerate, TRUE, TRUE, 3);


      hbox_pict_size = gtk_hbox_new (FALSE, 0);

      GtkWidget *label_pict_size = gtk_label_new("Picture size");
      gtk_box_pack_start (GTK_BOX (hbox_pict_size), label_pict_size, FALSE, FALSE, 0);

      opt_pict_size = gtk_option_menu_new ();
      menu_pict_size = gtk_menu_new ();
      set_active_pict_size_idx();

      for(i=0; i < pict_sizeArrN; i++) {
        GtkWidget *item = gtk_menu_item_new_with_label (pict_sizeArr[i]);
        g_signal_connect (G_OBJECT (item), "activate",
          G_CALLBACK(cb_update_menu_select_pict_size), opt_pict_size);
        gtk_menu_shell_append (GTK_MENU_SHELL (menu_pict_size), item);
      }

      gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_pict_size), menu_pict_size);

      gtk_option_menu_set_history (GTK_OPTION_MENU (opt_pict_size), active_pict_size_idx);

      gtk_box_pack_start (GTK_BOX (hbox_pict_size), opt_pict_size, FALSE, FALSE, 10);

      gtk_box_pack_start (GTK_BOX (vbox), hbox_pict_size, TRUE, TRUE, 0);

      GtkWidget *separator_pict_size = gtk_hseparator_new ();

      gtk_box_pack_start (GTK_BOX (vbox), separator_pict_size, TRUE, TRUE, 3);
    }

    GtkWidget *hbox_ok = gtk_hbox_new (TRUE, 0);

    GtkWidget *button_ok = gtk_button_new_with_label("OK");
    gtk_box_pack_start (GTK_BOX (hbox_ok), button_ok, TRUE, TRUE, 10);
    GtkWidget *button_cancel = gtk_button_new_with_label("Cancel");
    gtk_box_pack_start (GTK_BOX (hbox_ok), button_cancel, TRUE, TRUE, 10);

    g_signal_connect (GTK_OBJECT (button_ok), "clicked", G_CALLBACK (cb_ok), NULL);
    g_signal_connect (GTK_OBJECT (button_cancel), "clicked", G_CALLBACK (cb_cancel), NULL);

    gtk_box_pack_start (GTK_BOX (vbox), hbox_ok, TRUE, TRUE, 10);


    gtk_container_add (GTK_CONTAINER (codec_window), vbox);

    char *codec = NULL;

    if (fftv_mode & FFTV_MODE_TV)
      codec = get_arg_val("-vcodec");

    gtk_widget_show_all (codec_window);

    set_format_codec_active_idx(file_format, codec);
}

