#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "gtk/gtk.h"
#include "cron.h"
#include "tv-freq.h"
#include "fftv.h"

void destroy_prog_list_window();

static GtkWidget *prog_list_window = NULL;
static GtkWidget *sw;
GtkWidget *cron_treeview;
static GtkWidget *opt_av_source;
static GtkWidget *spinner_channel;
static GtkWidget *entry_program;
static GtkWidget *entry_dayweek;
static GtkWidget *spinner_month;
static GtkWidget *spinner_daymonth;
static GtkWidget *spinner_hour;
static GtkWidget *spinner_minute;
static GtkWidget *spinner_duration;
static GtkWidget *check_button_record;

static char *av_source_str[]={"tv", "radio"};
int av_source_strN = sizeof(av_source_str)/ sizeof(av_source_str[0]);


GArray *cr_entries = NULL;

static int qcmp_channel(const void *aa, const void *bb)
{
  TcronItem *a=(TcronItem *)aa, *b=(TcronItem *)bb;

  int d;

  if ((d = strcmp(a->av_source, b->av_source)))
    return d;

  if ((d = a->channel - b->channel))
    return d;

  if ((d = a->hour - b->hour))
    return d;

  return a->minute - b->minute;
}

void add_tcron_entry(TcronItem *ti)
{
//  dbg("add_tcron_ti:%x\n", ti);
  TcronItem t = *ti;

  t.av_source = g_strdup(ti->av_source);
  t.program = g_strdup(ti->program);
  t.dayweek = g_strdup(ti->dayweek);
  t.month = g_strdup(ti->month);
  t.daymonth = g_strdup(ti->daymonth);
  t.editable = FALSE;

  g_array_append_vals (cr_entries, &t, 1);
}


void load_program_entries_from_file();

static void add_tcron_entries (TcronItem *ti)
{

  printf("add_tcron_items a ....\n");
  g_return_if_fail (cr_entries != NULL);

  load_program_entries_from_file();
  g_array_sort (cr_entries,qcmp_channel);

  if (ti)
    add_tcron_entry(ti);

  printf("add_tcron_items b ....\n");
}


static GtkTreeModel *create_cron_model (TcronItem *ti)
{
  gint i = 0;
  GtkTreeIter iter;
  GtkListStore *model;

  printf("create_model ....\n");

  /* create array */
  cr_entries = g_array_sized_new (FALSE, FALSE, sizeof (TcronItem), 1);

  add_tcron_entries (ti);

  /* create list store */
  model = gtk_list_store_new (NUM_COLUMNS,
                              G_TYPE_STRING,              // av_source
                              G_TYPE_INT, G_TYPE_STRING,  // channel, program
                              G_TYPE_STRING,G_TYPE_STRING,   // dayweek, month
                              G_TYPE_STRING, G_TYPE_INT,   // daymonth, hour
                              G_TYPE_INT, G_TYPE_INT,    // minute duration
                              G_TYPE_BOOLEAN,            // record
                              G_TYPE_BOOLEAN);

  /* add items */
  for (i = 0; i < cr_entries->len; i++) {
      gtk_list_store_append (model, &iter);

      gtk_list_store_set (model, &iter,
                          COLUMN_AV_SOURCE,
                          g_array_index (cr_entries, TcronItem, i).av_source,
                          COLUMN_CHANNEL,
                          g_array_index (cr_entries, TcronItem, i).channel,
                          COLUMN_PROGRAM,
                          g_array_index (cr_entries, TcronItem, i).program,
                          COLUMN_DAYWEEK,
                          g_array_index (cr_entries, TcronItem, i).dayweek,
                          COLUMN_MONTH,
                          g_array_index (cr_entries, TcronItem, i).month,
                          COLUMN_DAYMONTH,
                          g_array_index (cr_entries, TcronItem, i).daymonth,
                          COLUMN_HOUR,
                          g_array_index (cr_entries, TcronItem, i).hour,
                          COLUMN_MINUTE,
                          g_array_index (cr_entries, TcronItem, i).minute,
                          COLUMN_DURATION,
                          g_array_index (cr_entries, TcronItem, i).duration,
                          COLUMN_RECORD,
                          g_array_index (cr_entries, TcronItem, i).record,
                          COLUMN_EDITABLE,
                          g_array_index (cr_entries, TcronItem, i).editable,
                          -1);

  }

  return GTK_TREE_MODEL (model);
}




static void
add_cron_columns (GtkTreeView *cron_treeview)
{
  GtkCellRenderer *renderer;

  printf("add_columns a\n");

  /* av_source column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_AV_SOURCE);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "AV source", renderer,
                                               "text", COLUMN_AV_SOURCE,
                                               NULL);

  /* channel column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_CHANNEL);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Ch #", renderer,
                                               "text", COLUMN_CHANNEL,
                                               NULL);

  /* program column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_PROGRAM);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Program Name", renderer,
                                               "text", COLUMN_PROGRAM,
                                               NULL);

  /* dayweek column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_DAYWEEK);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Wk Days", renderer,
                                               "text", COLUMN_DAYWEEK,
                                               NULL);

  /* month column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_MONTH);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Month", renderer,
                                               "text", COLUMN_MONTH,
                                               NULL);

  /* daymonth column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_DAYMONTH);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Mo day", renderer,
                                               "text", COLUMN_DAYMONTH,
                                               NULL);


  /* hour column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_HOUR);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Hr", renderer,
                                               "text", COLUMN_HOUR,
                                               NULL);

  /* minute column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_MINUTE);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Mi", renderer,
                                               "text", COLUMN_MINUTE,
                                               NULL);

  /* duration column */
  renderer = gtk_cell_renderer_text_new ();
  g_object_set_data (G_OBJECT (renderer), "column", (gint *)COLUMN_DURATION);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Duration", renderer,
                                               "text", COLUMN_DURATION,
                                               NULL);

  // record column
  renderer = gtk_cell_renderer_toggle_new ();
  g_object_set (G_OBJECT (renderer), "xalign", 0.0, NULL);

  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (cron_treeview),
                                               -1, "Record",
                                               renderer,
                                               "active", COLUMN_RECORD,
                                               NULL);
}


void cb_opt_av_source_menu_select(GtkWidget  *item,  GtkWidget *opt)
{
   int idx = gtk_option_menu_get_history(GTK_OPTION_MENU(opt));

   printf("select %s\n", av_source_str[idx]);

  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  gchar *old_text;
  gtk_tree_model_get (model, &iter, COLUMN_AV_SOURCE, &old_text, -1);
  g_free(old_text);

  g_array_index (cr_entries, TcronItem, i).av_source = g_strdup(av_source_str[idx]);

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_AV_SOURCE,
      g_array_index (cr_entries, TcronItem, i).av_source, -1);
}




void set_active_opt_av_source_item(char *s)
{
  int i;

  for(i=0; i < av_source_strN; i++)
    if (!strcmp(av_source_str[i], s))
      break;

  if (i==av_source_strN) {
    printf("unknown item %s\n", s);
    exit(-1);
  }

  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_av_source), i);
}

void set_spinner_channel(int rno)
{
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner_channel), (double)rno);
}

void set_entry_program(char *s)
{
  gtk_entry_set_text(GTK_ENTRY(entry_program), s);
}

void set_entry_dayweek(char *s)
{
  gtk_entry_set_text(GTK_ENTRY(entry_dayweek), s);
}


void set_spinner_month(char *month)
{
  int value;

  if (!strcmp(month,"*"))
    value = 0;
  else
    value = atof(month);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner_month), value);
}


void set_spinner_daymonth(char *daymonth)
{
  int value;

  if (!strcmp(daymonth,"*"))
    value = 0;
  else
    value = atof(daymonth);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner_daymonth), value);
}


void set_spinner_hour(int hour)
{
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner_hour), (double)hour);
}


void set_spinner_minute(int minute)
{
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner_minute), (double)minute);
}


void set_spinner_duration(int duration)
{
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner_duration), (double)duration);
}


void set_check_button_record(gboolean val)
{
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_record), val);
}



static void callback_row_selected(GtkTreeSelection *treeselection,
                                  gpointer user_data)
{
  GtkTreeIter iter;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  dbg("row selected\n");

  if (!gtk_tree_selection_get_selected (treeselection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  char *av_source=g_array_index (cr_entries, TcronItem, i).av_source;
  set_active_opt_av_source_item(av_source);

  int channel=g_array_index (cr_entries, TcronItem, i).channel;
  set_spinner_channel(channel);

  char *program = g_array_index (cr_entries, TcronItem, i).program;
  set_entry_program(program);

  char *dayweek = g_array_index (cr_entries, TcronItem, i).dayweek;
  set_entry_dayweek(dayweek);

  char *month = g_array_index (cr_entries, TcronItem, i).month;
  set_spinner_month(month);

  char *daymonth = g_array_index (cr_entries, TcronItem, i).daymonth;
  set_spinner_daymonth(daymonth);

  int hour = g_array_index (cr_entries, TcronItem, i).hour;
  set_spinner_hour(hour);

  int minute = g_array_index (cr_entries, TcronItem, i).minute;
  set_spinner_minute(minute);

  int duration = g_array_index (cr_entries, TcronItem, i).duration;
  set_spinner_duration(duration);

  gboolean record = g_array_index (cr_entries, TcronItem, i).record;
  set_check_button_record(record);


  printf("selected %s %s %d dayweek:%s mo:%s moday%s hour:%d min:%d record:%d\n",
    av_source, program, channel, dayweek,
    month, daymonth, hour, minute, record);
}

void cb_check_button_toggled_record(GtkToggleButton *togglebutton,
                                            gpointer user_data)
{
  gboolean val = gtk_toggle_button_get_active(togglebutton);

  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  g_array_index (cr_entries, TcronItem, i).record = val;

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_RECORD, val, -1);

//  printf("val:%d\n",val);
}

void cb_spinner_value_changed_channel(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
  double value = gtk_spin_button_get_value(spinbutton);
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
  gint i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  g_array_index (cr_entries, TcronItem, i).channel = value;

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_CHANNEL,
      g_array_index (cr_entries, TcronItem, i).channel, -1);
}


void cb_spinner_value_changed_month(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
  double value = gtk_spin_button_get_value(spinbutton);
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
  gint i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  char ostr[16];

  if (value > 0.0) {
    sprintf(ostr, "%d", (int)value);
    set_entry_dayweek("*");
    copy_dayweek_entry_data(entry_dayweek);
  }
  else
    strcpy(ostr,"*");

  gchar *old_text;
  gtk_tree_model_get (model, &iter, COLUMN_MONTH, &old_text, -1);
  g_free(old_text);

  g_array_index (cr_entries, TcronItem, i).month = g_strdup(ostr);

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_MONTH,
      g_array_index (cr_entries, TcronItem, i).month, -1);
}


void cb_spinner_value_changed_daymonth(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
  double value = gtk_spin_button_get_value(spinbutton);
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
  gint i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  char ostr[16];

  if (value > 0.0)
    sprintf(ostr, "%d", (int)value);
  else
    strcpy(ostr,"*");

  gchar *old_text;
  gtk_tree_model_get (model, &iter, COLUMN_DAYMONTH, &old_text, -1);
  g_free(old_text);

  g_array_index (cr_entries, TcronItem, i).daymonth = g_strdup(ostr);

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_DAYMONTH,
      g_array_index (cr_entries, TcronItem, i).daymonth, -1);
}


void cb_spinner_value_changed_hour(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
  double value = gtk_spin_button_get_value(spinbutton);
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
  gint i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  g_array_index (cr_entries, TcronItem, i).hour = value;

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_HOUR,
      g_array_index (cr_entries, TcronItem, i).hour, -1);
}


void cb_spinner_value_changed_minute(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
  double value = gtk_spin_button_get_value(spinbutton);
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
  gint i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  g_array_index (cr_entries, TcronItem, i).minute = value;

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_MINUTE,
      g_array_index (cr_entries, TcronItem, i).minute, -1);
}


void cb_spinner_value_changed_duration(GtkSpinButton *spinbutton,
                                      gpointer user_data)
{
  double value = gtk_spin_button_get_value(spinbutton);
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
  gint i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  g_array_index (cr_entries, TcronItem, i).duration = value;

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_DURATION,
      g_array_index (cr_entries, TcronItem, i).duration, -1);
}



void copy_program_entry_data(GtkEntry *entry)
{
  const gchar *newtext = gtk_entry_get_text (entry);
  printf("newtext:%s\n",newtext);

  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  gchar *old_text;
  gtk_tree_model_get (model, &iter, COLUMN_PROGRAM, &old_text, -1);
  g_free (old_text);

  g_array_index (cr_entries, TcronItem, i).program = g_strdup(newtext);

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_PROGRAM, newtext, -1);
}


void cb_entry_program(GtkEntry *entry, gpointer user_data)
{
  copy_program_entry_data(entry);
}


gboolean cb_entry_focus_out_program(GtkEntry *entry, GdkEventFocus *event, gpointer user_data)
{
  copy_program_entry_data(entry);
  return FALSE;
}


void copy_dayweek_entry_data(GtkEntry *entry)
{
  const gchar *newtext = gtk_entry_get_text (entry);
  printf("newtext:%s\n",newtext);

  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  gint i;
  GtkTreePath *path;

  path = gtk_tree_model_get_path (model, &iter);
  i = gtk_tree_path_get_indices (path)[0];
  gtk_tree_path_free(path);

  gchar *old_text;
  gtk_tree_model_get (model, &iter, COLUMN_DAYWEEK, &old_text, -1);
  g_free (old_text);

  g_array_index (cr_entries, TcronItem, i).dayweek = g_strdup(newtext);

  gtk_list_store_set (GTK_LIST_STORE (model), &iter, COLUMN_DAYWEEK, newtext, -1);
}


void cb_entry_dayweek(GtkEntry *entry, gpointer user_data)
{
  copy_dayweek_entry_data(entry);
}


gboolean cb_entry_focus_out_dayweek(GtkEntry *entry, GdkEventFocus *event, gpointer user_data)
{
  copy_dayweek_entry_data(entry);
  return FALSE;
}



static void scroll_to_bottom()
{
  GtkTreeIter iter, iter_last;
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  if (!gtk_tree_model_get_iter_first(model, &iter))
    return;
#if 0
  if (cr_entries->len == 1) {
    GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
    dbg("selection:%x  xxxx\n", selection);
    gtk_tree_selection_select_iter(selection, &iter);
    callback_row_selected(selection, NULL); // bug
    return;
  }
#endif

  do {
    iter_last = iter;
  } while (gtk_tree_model_iter_next(model, &iter));

  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  dbg("selection:%x\n", selection);
  gtk_tree_selection_select_iter(selection, &iter_last);
  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(cron_treeview),
     gtk_tree_model_get_path(gtk_tree_view_get_model(GTK_TREE_VIEW(cron_treeview)),&iter_last),
        NULL,FALSE,0.0,0.0);
}

static void cb_button_delete_clicked(GtkWidget *widget, gpointer   data)
{
  GtkTreeSelection *selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  GtkTreeModel *model = gtk_tree_view_get_model (GTK_TREE_VIEW(cron_treeview));

  GtkTreeIter iter;

  if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
    return;

  GtkTreePath *path;

  path = gtk_tree_model_get_path (model, &iter);
  gtk_tree_path_free(path);

  gchar *old_text;

  gtk_tree_model_get (model, &iter, COLUMN_AV_SOURCE, &old_text, -1);
  g_free(old_text);
  gtk_tree_model_get (model, &iter, COLUMN_PROGRAM, &old_text, -1);
  g_free(old_text);
  gtk_tree_model_get (model, &iter, COLUMN_DAYWEEK, &old_text, -1);
  g_free(old_text);
  gtk_tree_model_get (model, &iter, COLUMN_MONTH, &old_text, -1);
  g_free(old_text);
  gtk_tree_model_get (model, &iter, COLUMN_DAYMONTH, &old_text, -1);
  g_free(old_text);

  gtk_list_store_remove (GTK_LIST_STORE(model), &iter);

  GtkTreeIter iter_first;
  if (!gtk_tree_model_get_iter_first(model, &iter_first))
    return;

  if (!gtk_list_store_iter_is_valid(GTK_LIST_STORE(model), &iter)) {
    scroll_to_bottom();
    return;
  }

  selection=gtk_tree_view_get_selection(GTK_TREE_VIEW(cron_treeview));
  gtk_tree_selection_select_iter(selection, &iter);
  gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(cron_treeview),
     gtk_tree_model_get_path(gtk_tree_view_get_model(GTK_TREE_VIEW(cron_treeview)),&iter),
        NULL,FALSE,0.0,0.0);

  printf("button_delete_clicked\n");
}


void save_program_entries_to_file();
void cb_button_submit_clicked()
{
  printf("....... cb_button_submit_clicked\n");
  save_program_entries_to_file();
  destroy_prog_list_window();
}

static void callback_win_delete();



void create_prog_list_window(TcronItem *ti)
{
  /* create prog_list_window, etc */
  prog_list_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  gtk_window_set_title (GTK_WINDOW (prog_list_window), "Program reminder/recorder");
  GtkWidget *win_icon=gtk_image_new_from_file(ICON_BASENAME "/fftv.png");
  gtk_window_set_icon(GTK_WINDOW(prog_list_window),gtk_image_get_pixbuf(GTK_IMAGE(win_icon)));

  gtk_container_set_border_width (GTK_CONTAINER (prog_list_window), 5);

  g_signal_connect (G_OBJECT (prog_list_window), "destroy",
                    G_CALLBACK (gtk_widget_destroyed), &prog_list_window);

  g_signal_connect (G_OBJECT (prog_list_window), "delete_event",
                      G_CALLBACK (callback_win_delete), NULL);


  GtkWidget *vbox = gtk_vbox_new (FALSE, 5);
  gtk_container_add (GTK_CONTAINER (prog_list_window), vbox);


  gtk_box_pack_start (GTK_BOX (vbox),
                      gtk_label_new ("TV/radio program reminder/recorder"),
                      FALSE, FALSE, 0);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
                                       GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
                                  GTK_POLICY_AUTOMATIC,
                                  GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox), sw, TRUE, TRUE, 0);



  /* create model */
  GtkTreeModel *model = create_cron_model (ti);

  /* create tree view */
  cron_treeview = gtk_tree_view_new_with_model (model);
  g_object_unref (G_OBJECT (model));
  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (cron_treeview), TRUE);

  GtkTreeSelection *tree_selection =
     gtk_tree_view_get_selection (GTK_TREE_VIEW (cron_treeview));
  gtk_tree_selection_set_mode (tree_selection, GTK_SELECTION_SINGLE);

  g_signal_connect (G_OBJECT(tree_selection), "changed",
                    G_CALLBACK (callback_row_selected), NULL);

  add_cron_columns (GTK_TREE_VIEW (cron_treeview));

  gtk_container_add (GTK_CONTAINER (sw), cron_treeview);

  gtk_window_set_default_size (GTK_WINDOW (prog_list_window), 600, 300);

  g_signal_connect (G_OBJECT (prog_list_window), "delete_event",
                    G_CALLBACK (gtk_main_quit), NULL);

  GtkWidget *hbox_edit_top = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox_edit_top , FALSE, FALSE, 0);

  GtkWidget *frame = gtk_frame_new("Editing");
  gtk_box_pack_start (GTK_BOX (hbox_edit_top), frame, FALSE, FALSE, 0);

  GtkWidget *vbox_delete = gtk_vbox_new (FALSE, 5);

  GtkWidget *button_delete = gtk_button_new();
  g_signal_connect (G_OBJECT (button_delete), "clicked", G_CALLBACK (cb_button_delete_clicked), NULL);
  gtk_box_pack_start (GTK_BOX (vbox_delete), button_delete, TRUE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox_edit_top), vbox_delete, FALSE, FALSE, 0);

  GtkWidget *hbox_button_delete = gtk_hbox_new (FALSE, 5);

  gtk_container_add (GTK_CONTAINER (button_delete), hbox_button_delete);

  GtkWidget *image_delete = gtk_image_new_from_file ("/usr/share/pixmaps/no.xpm");
  gtk_box_pack_start (GTK_BOX (hbox_button_delete), image_delete, TRUE, FALSE, 0);
  GtkWidget *label_delete = gtk_label_new("Delete");
  gtk_box_pack_start (GTK_BOX (hbox_button_delete), label_delete, TRUE, FALSE, 0);

  GtkWidget *vbox_edit = gtk_vbox_new (FALSE, 5);

  gtk_container_add (GTK_CONTAINER (frame), vbox_edit);

  GtkWidget *hbox_edit1 = gtk_hbox_new (FALSE, 5);

  gtk_box_pack_start (GTK_BOX (vbox_edit), hbox_edit1, FALSE, FALSE, 0);


  GtkWidget *vbox_av_source = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_av_source = gtk_label_new("AV_source");
  gtk_box_pack_start (GTK_BOX (vbox_av_source), label_av_source, FALSE, FALSE, 0);


  opt_av_source = gtk_option_menu_new ();

  GtkWidget *menu_av_source = gtk_menu_new ();

  int i;
  for(i=0; i < av_source_strN; i++) {
    GtkWidget *item = gtk_menu_item_new_with_label (av_source_str[i]);
    g_signal_connect (G_OBJECT (item), "activate", G_CALLBACK(cb_opt_av_source_menu_select),
                      opt_av_source);
    gtk_menu_shell_append (GTK_MENU_SHELL (menu_av_source), item);
  }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_av_source), menu_av_source);

  gtk_box_pack_start (GTK_BOX (vbox_av_source), opt_av_source, FALSE, FALSE, 3);

  gtk_box_pack_start (GTK_BOX (hbox_edit1), vbox_av_source, FALSE, FALSE, 0);

  GtkWidget *vbox_channel = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_channel = gtk_label_new("Channel #");
  gtk_box_pack_start (GTK_BOX (vbox_channel), label_channel, FALSE, FALSE, 0);


  double step = 1.0;
  GtkAdjustment *adj_channel = (GtkAdjustment *) gtk_adjustment_new (
       33.0, 1.0, 200.0, step, step, step);
  spinner_channel = gtk_spin_button_new (adj_channel, step, 0);

  g_signal_connect (G_OBJECT(spinner_channel), "value-changed",
                    G_CALLBACK(cb_spinner_value_changed_channel), NULL);

  gtk_widget_set_size_request (spinner_channel, 60, -1);

  gtk_box_pack_start (GTK_BOX (vbox_channel), spinner_channel, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox_edit1), vbox_channel, FALSE, FALSE, 0);


  GtkWidget *vbox_program = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_program = gtk_label_new("Program Name");
  gtk_box_pack_start (GTK_BOX (vbox_program), label_program, FALSE, FALSE, 0);

  entry_program = gtk_entry_new_with_max_length (128);
  g_signal_connect (G_OBJECT(entry_program), "activate",
                    G_CALLBACK(cb_entry_program), NULL);
  g_signal_connect (G_OBJECT(entry_program), "focus-out-event",
                    G_CALLBACK(cb_entry_program), NULL);

  gtk_entry_set_width_chars(GTK_ENTRY (entry_program), 12);
  gtk_box_pack_start (GTK_BOX (vbox_program), entry_program, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit1), vbox_program, FALSE, FALSE, 0);

  GtkWidget *hbox_edit2 = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX (vbox_edit), hbox_edit2, FALSE, FALSE, 0);

  GtkWidget *vbox_dayweek = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_dayweek = gtk_label_new("Week days");
  gtk_box_pack_start (GTK_BOX (vbox_dayweek), label_dayweek, FALSE, FALSE, 0);

  entry_dayweek = gtk_entry_new_with_max_length (40);
  GtkTooltips *entry_dayweek_tips = gtk_tooltips_new ();
  gtk_tooltips_set_tip (GTK_TOOLTIPS (entry_dayweek_tips), entry_dayweek, "0 to 6, * for all, 1-5 (Mon thr Fri)",NULL);

  g_signal_connect (G_OBJECT(entry_dayweek), "activate",
                    G_CALLBACK(cb_entry_dayweek), NULL);
  g_signal_connect (G_OBJECT(entry_dayweek), "focus-out-event",
                    G_CALLBACK(cb_entry_dayweek), NULL);

  gtk_entry_set_width_chars(GTK_ENTRY (entry_dayweek), 11);
  gtk_box_pack_start (GTK_BOX (vbox_dayweek), entry_dayweek, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit2), vbox_dayweek, FALSE, FALSE, 0);


  GtkWidget *vbox_month = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_month = gtk_label_new("Month");
  gtk_box_pack_start (GTK_BOX (vbox_month), label_month, FALSE, FALSE, 0);


  GtkAdjustment *adj_month = (GtkAdjustment *) gtk_adjustment_new (
       33.0, 0.0, 12.0, step, step, step);
  spinner_month = gtk_spin_button_new (adj_month, step, 0);
  g_signal_connect (G_OBJECT(spinner_month), "value-changed",
                    G_CALLBACK(cb_spinner_value_changed_month), NULL);
  gtk_widget_set_size_request (spinner_month, 40, -1);
  gtk_box_pack_start (GTK_BOX (vbox_month), spinner_month, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit2), vbox_month, FALSE, FALSE, 0);


  GtkWidget *vbox_daymonth = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_daymonth = gtk_label_new("Mo day");
  gtk_box_pack_start (GTK_BOX (vbox_daymonth), label_daymonth, FALSE, FALSE, 0);

  GtkAdjustment *adj_daymonth = (GtkAdjustment *) gtk_adjustment_new (
       31.0, 0.0, 31.0, step, step, step);
  spinner_daymonth = gtk_spin_button_new (adj_daymonth, step, 0);
  g_signal_connect (G_OBJECT(spinner_daymonth), "value-changed",
                    G_CALLBACK(cb_spinner_value_changed_daymonth), NULL);
  gtk_widget_set_size_request (spinner_daymonth, 40, -1);
  gtk_box_pack_start (GTK_BOX (vbox_daymonth), spinner_daymonth, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit2), vbox_daymonth, FALSE, FALSE, 0);


  GtkWidget *vbox_hour = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_hour = gtk_label_new("Hour");
  gtk_box_pack_start (GTK_BOX (vbox_hour), label_hour, FALSE, FALSE, 0);

  GtkAdjustment *adj_hour = (GtkAdjustment *) gtk_adjustment_new (
       33.0, 0.0, 23.0, step, step, step);
  spinner_hour = gtk_spin_button_new (adj_hour, step, 0);
  g_signal_connect (G_OBJECT(spinner_hour), "value-changed",
                    G_CALLBACK(cb_spinner_value_changed_hour), NULL);
  gtk_widget_set_size_request (spinner_hour, 40, -1);
  gtk_box_pack_start (GTK_BOX (vbox_hour), spinner_hour, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit2), vbox_hour, FALSE, FALSE, 0);


  GtkWidget *vbox_minute = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_minute = gtk_label_new("Minute");
  gtk_box_pack_start (GTK_BOX (vbox_minute), label_minute, FALSE, FALSE, 0);

  GtkAdjustment *adj_minute = (GtkAdjustment *) gtk_adjustment_new (
       33.0, 0.0, 59.0, step, step, step);
  spinner_minute = gtk_spin_button_new (adj_minute, step, 0);
  g_signal_connect (G_OBJECT(spinner_minute), "value-changed",
                    G_CALLBACK(cb_spinner_value_changed_minute), NULL);
  gtk_widget_set_size_request (spinner_minute, 40, -1);
  gtk_box_pack_start (GTK_BOX (vbox_minute), spinner_minute, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit2), vbox_minute, FALSE, FALSE, 0);


  GtkWidget *vbox_duration = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_duration = gtk_label_new("Duration(Mi)");
  gtk_box_pack_start (GTK_BOX (vbox_duration), label_duration, FALSE, FALSE, 0);

  GtkAdjustment *adj_duration = (GtkAdjustment *) gtk_adjustment_new (
       33.0, 1.0, 999.0, step, step, step);
  spinner_duration = gtk_spin_button_new (adj_duration, step, 0);
  g_signal_connect (G_OBJECT(spinner_duration), "value-changed",
                    G_CALLBACK(cb_spinner_value_changed_duration), NULL);
  gtk_widget_set_size_request (spinner_duration, 40, -1);
  gtk_box_pack_start (GTK_BOX (vbox_duration), spinner_duration, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit2), vbox_duration, FALSE, FALSE, 0);



  GtkWidget *vbox_record = gtk_vbox_new (FALSE, 2);
  GtkWidget *label_record = gtk_label_new("Record");

  gtk_box_pack_start (GTK_BOX (vbox_record), label_record, FALSE, FALSE, 0);

  check_button_record = gtk_check_button_new ();
  g_signal_connect (G_OBJECT (check_button_record), "toggled",
                    G_CALLBACK (cb_check_button_toggled_record), NULL);


  gtk_box_pack_start (GTK_BOX (vbox_record), check_button_record, FALSE, FALSE, 0);

  gtk_box_pack_start (GTK_BOX (hbox_edit2), vbox_record, FALSE, FALSE, 0);


  GtkWidget *hbox = gtk_hbox_new (FALSE, 5);

  GtkWidget *button_submit = gtk_button_new_with_label("Submit");

  g_signal_connect (G_OBJECT (button_submit), "clicked", G_CALLBACK (cb_button_submit_clicked), NULL);
  GtkWidget *button_restore = gtk_button_new_with_label("Restore");
  g_signal_connect (G_OBJECT (button_restore), "clicked", G_CALLBACK (callback_win_delete), NULL);

  gtk_box_pack_start (GTK_BOX (hbox), button_submit, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), button_restore, TRUE, TRUE, 0);

  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);

  gtk_widget_show_all (prog_list_window);


  scroll_to_bottom();
}

void create_prog_list_window_current_channel()
{
  TcronItem ti;
  struct tm ltm;

  time_t ltime=time(NULL);
  localtime_r(&ltime, &ltm);

  ti.av_source = (fftv_mode & FFTV_MODE_RADIO) ? "radio" : "tv";
  ti.channel = pchan_current->rno;
  ti.program = "untitled";
  char tt[16];
  sprintf(tt,"%d", ltm.tm_wday);
  ti.dayweek = tt;

  ti.month = "*";
  ti.daymonth = "*";
  ti.hour = ltm.tm_hour;
  ti.minute = 0;
  ti.duration = 60;
  ti.record = FALSE;
  ti.editable = FALSE;

  create_prog_list_window(&ti);
}


void create_prog_list_window_view()
{
  create_prog_list_window(NULL);
}


void destroy_prog_list_window()
{
  int i;

  gtk_widget_destroy(prog_list_window);

  dbg("cr_entries->len:%d\n", cr_entries->len);
  for (i = 0; i < cr_entries->len; i++) {
    g_free(g_array_index (cr_entries, TcronItem, i).av_source);
    g_free(g_array_index (cr_entries, TcronItem, i).program);
    g_free(g_array_index (cr_entries, TcronItem, i).dayweek);
    g_free(g_array_index (cr_entries, TcronItem, i).month);
    g_free(g_array_index (cr_entries, TcronItem, i).daymonth);
  }

  g_array_free(cr_entries, TRUE);
}


static void callback_win_delete()
{
  dbg("callback_win_delete ..................\n");
  destroy_prog_list_window();
  gtk_main_quit();
}
