#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "fftv.h"
#include "reminder.h"

char *get_fftv_sock_path();

static int my_system(char *cmd)
{
  int pid;

  if (!(pid=fork())) {
    char *shell = "/bin/sh";

    execl(shell,shell,"-c",cmd, NULL);
  } else
  if (pid > 0) {
    return pid;
  } else
    fprintf(stderr,"cannot fork");

  return 0;
}


static int connect_rem_srv()
{
  int sockfd, servlen;
  struct sockaddr_un serv_addr;

  bzero((char *) &serv_addr,sizeof(serv_addr));

  serv_addr.sun_family = AF_UNIX;
  strcpy(serv_addr.sun_path,get_fftv_sock_path());
  servlen = strlen(serv_addr.sun_path) + sizeof(serv_addr.sun_family);

  if ((sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
    perror("cannot open socket");
    exit(-1);
  }

  if (connect(sockfd, (struct sockaddr *) &serv_addr, servlen) < 0) {
    close(sockfd);
    return -1;
  }
  return sockfd;
}

void init_freq_dir();

int main(int argc, char **argv)
{
  int sockfd;
  char *prog = basename(argv[0]);
  int pid;
  int b_ffrecord=0;


  if (argc < 3) {
    fprintf(stderr, "%s for fftv version %s\n",prog, FFTV_VERSION);
    fprintf(stderr,"Usage:\n\t%s [radio|tv] tv-radio-channel tv-program-name [time_period(minutes)]\n", prog);
    exit(-1);
  }

  if (!strcmp(prog, "ffrecord")) {
    b_ffrecord = 1;
    if (argc < 5) {
      fprintf(stderr,"must specify recoring time\n");
      exit(-1);
    }

    int minutes = atoi(argv[4]);

    if (minutes <= 0)
      fprintf(stderr,"invalid recoring time %s\n",argv[4]);

    char tt[256];

    sprintf(tt,"FF_RECORDING_TIME=%d",minutes * 60);
    putenv(tt);
  }

  REMINDER remind;
  bzero(&remind,sizeof(REMINDER));

  strcpy(remind.channel_name, argv[2]);

  if ((sockfd=connect_rem_srv())<0) {
    char cmd[128];
    char last_fname[128];

    init_freq_dir();

    if (!strcmp(argv[1],"radio")) {
      strcpy(cmd,"ffradio");
      fftv_mode = FFTV_MODE_RADIO;
    }
    else {
      strcpy(cmd,"fftv");
      fftv_mode = FFTV_MODE_TV;
    }

    save_channel_name_last_session(remind.channel_name);

    pid = my_system(cmd);
    sleep(1);
    if ((sockfd=connect_rem_srv())<0) {
      fprintf(stderr,"fftv is not running. \n");
      exit(-1);
    }
  }

//  printf("connect fd %d\n",sockfd);


  if (!strcmp(argv[1],"radio"))
    remind.fftv_mode = FFTV_MODE_RADIO;
  else
    remind.fftv_mode = FFTV_MODE_TV;



  if (argc==4)
    strcpy(remind.program_name, argv[3]);

  write(sockfd,&remind,sizeof(REMINDER));

  close(sockfd);

  int status;
//  if (b_ffrecord)
  {
    printf("........... wait...\n");
    wait(&status);
  }

  return status;
}
