#include <glib.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "tv-freq.h"
#include "tv-win.h"

static GtkWidget *tv_disp_opt_window = NULL;

typedef struct {
  char *name;
  TV_WIN_DISP_MODE mode;
} DISP_OPT;

static DISP_OPT disp_opts[] =
  {
    {"Overlay", TV_OVERLAY},
    {"Grab/XVideo", TV_GRAB_XV}
  };

static int disp_optsN = sizeof(disp_opts) / sizeof(disp_opts[0]);


static DISP_OPT disp_opts_rec[] =
  {
    {"Overlay", TV_OVERLAY_RECORD},
    {"Grab/XVideo", TV_GRAB_XV_RECORD}
  };

static int disp_opts_recN = sizeof(disp_opts_rec) / sizeof(disp_opts_rec[0]);

static int disp_mode, disp_mode_rec;

char *get_fftv_states_dir(char *fftv_states_dir);
char *disp_opt_file_name()
{
  static char filename[256];

  get_fftv_states_dir(filename);
  strcat(filename,"/tv-disp-option");

  return filename;
}

void save_disp_opt()
{
  char *file = disp_opt_file_name();

  FILE *fp;

  if ((fp=fopen(file, "w"))==NULL) {
    p_err("cannot write to file %s\n", file);
  }

  fprintf(fp,"%x", tv_win_disp_mode);
  fclose(fp);
}


void load_disp_opt()
{
  char *file = disp_opt_file_name();

  FILE *fp;

  dbg("load_disp_opt: %s\n", file);

  if (getenv("FFTV_DISP_GRAB")) {
    tv_win_disp_mode = TV_GRAB_XV | TV_GRAB_XV_RECORD;
    save_disp_opt();
    return;
  } else
  if ((fp=fopen(file, "r"))==NULL) {
    tv_win_disp_mode = TV_OVERLAY | TV_OVERLAY_RECORD;
    save_disp_opt();
    return;
  }

  int mode;

  fscanf(fp,"%x", &mode);
  tv_win_disp_mode = mode;
  fclose(fp);
}


static gboolean close_application( GtkWidget *widget,
                                   GdkEvent  *event,
                                   gpointer   data )
{
  gtk_widget_destroy(tv_disp_opt_window);
  tv_disp_opt_window = NULL;

  return FALSE;
}


static gboolean cb_ok( GtkWidget *widget, GdkEvent  *event, gpointer   data )
{
  if (TV_WIN_DISP_MODE_GRAB && (disp_mode & TV_OVERLAY)) {
    tv_win_refresh_mode = TV_WIN_REFRESH_OVERLAY;
    stop_refresh_grab_tv_win();
    tv_pic_on();
  } else
  if (!TV_WIN_DISP_MODE_GRAB && (disp_mode & TV_GRAB_XV)) {
    tv_win_refresh_mode = TV_WIN_REFRESH_GRAB;
    create_x_shm_image();
    tv_pic_off();
    normal_tv_win_frame_rate();
  }

  tv_win_disp_mode = disp_mode | disp_mode_rec;
  save_disp_opt();

  gtk_widget_destroy(tv_disp_opt_window);
  tv_disp_opt_window = NULL;

  return FALSE;
}


static void callback_button_clicked( GtkWidget *widget,  DISP_OPT *dopt)
{
  disp_mode = dopt->mode;
}


static void callback_button_rec_clicked( GtkWidget *widget, DISP_OPT *dopt)
{
  disp_mode_rec = dopt->mode;
}

extern GtkWidget *gtv_win;


void create_tv_dislay_option_window()
{
    GtkWidget *box_top, *box1, *box2;
    GtkWidget *button, *button_close;
    GtkWidget *frame1, *frame2;

    tv_disp_opt_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
#if 0
    gtk_window_set_modal(tv_disp_opt_window, TRUE);
    gtk_window_set_transient_for(GTK_WINDOW(tv_disp_opt_window), GTK_WINDOW(gtv_win));
#endif

    g_signal_connect (G_OBJECT (tv_disp_opt_window), "delete_event",
                      G_CALLBACK (close_application),
                      NULL);

    gtk_window_set_title (GTK_WINDOW (tv_disp_opt_window), "TV display options");
    gtk_container_set_border_width (GTK_CONTAINER (tv_disp_opt_window), 3);


    box_top = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (tv_disp_opt_window), box_top);

    frame1 = gtk_frame_new("Normal display");
    gtk_box_pack_start (GTK_BOX (box_top), frame1, TRUE, TRUE, 5);

    box1 = gtk_hbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame1), box1);


    dbg("tv_win_disp_mode:%x\n", tv_win_disp_mode);

    disp_mode = tv_win_disp_mode & 0xf;

    int i;
    GSList *group1=NULL;

    for(i=0;i< disp_optsN;i++) {
      DISP_OPT *dopt = disp_opts + i;

      button = gtk_radio_button_new_with_label (group1, dopt->name);
      gtk_box_pack_start (GTK_BOX (box1), button, TRUE, TRUE, 3);

      group1 = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));

      g_signal_connect (G_OBJECT (button), "clicked",
         G_CALLBACK (callback_button_clicked), dopt);

      if (tv_win_disp_mode & dopt->mode)
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
    }



    frame2 = gtk_frame_new("Recording display");
    gtk_box_pack_start (GTK_BOX (box_top), frame2, TRUE, TRUE, 5);

    box2 = gtk_hbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (frame2), box2);

    disp_mode_rec = tv_win_disp_mode & 0xf00;

    GSList *group2=NULL;

    for(i=0;i< disp_opts_recN;i++) {
      DISP_OPT *dopt = disp_opts_rec + i;

      button = gtk_radio_button_new_with_label (group2, dopt->name);
      gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 3);

      group2 = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));

      g_signal_connect (G_OBJECT (button), "clicked",
         G_CALLBACK (callback_button_rec_clicked), dopt);

      if (tv_win_disp_mode & dopt->mode) {
        dbg("set %s %x %x\n", dopt->name, tv_win_disp_mode, dopt->mode);
        gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
      }
    }


    button_close = gtk_button_new_with_label ("OK");
    g_signal_connect_swapped (G_OBJECT (button_close), "clicked",
                              G_CALLBACK (cb_ok),
                              G_OBJECT (tv_disp_opt_window));
    gtk_box_pack_start (GTK_BOX (box_top), button_close, TRUE, TRUE, 10);
    GTK_WIDGET_SET_FLAGS (button_close, GTK_CAN_DEFAULT);
#if 0
    gtk_widget_grab_default (button_close);
#endif
    gtk_widget_show_all (tv_disp_opt_window);
}
