/*
    Adapted from xawtv
 */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <linux/videodev.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include "gtk/gtk.h"
#include "tv-freq.h"
#include "fftv.h"
#include "vidstr.h"


CHANLIST  *chanlist;
int       chanlistN;
int *full_chan_arr, full_chan_arrN;    // remote control code arry

void set_channel_chan_idx();
extern void change_win_name(char *str);
void draw_channel_name(char *s, int clear);
void tv_audio_off();
void tv_audio_on();
void draw_radio_freq(float freq);
void draw_radio_station_name(char *s);

CHANLIST *pchan_current;

int chan_idx;
extern int vfd;
int xwin=0;

#define iabs(i) ((i) < 0 ? -(i):(i))
#define CHFREQ(i) (i*16/1000)


static int qcmp_rno(const void *aa, const void *bb)
{
  CHANLIST *a=(CHANLIST *)aa;
  CHANLIST *b=(CHANLIST *)bb;

  return a->rno - b->rno;
}

static void sort_chanlist_rno()
{
  qsort(chanlist,chanlistN,sizeof(CHANLIST),qcmp_rno);
}


static int qcmp_freq(const void *aa, const void *bb)
{
  CHANLIST *a=(CHANLIST *)aa;
  CHANLIST *b=(CHANLIST *)bb;

  return a->freq - b->freq;
}

static void sort_chanlist_freq()
{
  qsort(chanlist,chanlistN,sizeof(CHANLIST),qcmp_freq);
}


int chfreq(int freq)
{
   return (fftv_mode & FFTV_MODE_RADIO) ? freq_khz_radiodev(freq):CHFREQ(freq);
}

#define CH_CLEAR_SEC (2)
void draw_tv_ui_pcha(CHANLIST *pch);
void tv_set_selection_by_rno(int rno);
void radio_set_selection_by_name(gchar *name);

void disp_channel_name_pch(CHANLIST *pch)
{
    int rno=pch->rno;
    char st[32];

    if (fftv_mode & FFTV_MODE_RADIO) {
      draw_radio_freq((float)pch->freq/1000);
      draw_radio_station_name(pch->name);
      radio_set_selection_by_name(pch->name);
    }
    else {
      sprintf(st,"%d",rno);
      draw_channel_name(st,CH_CLEAR_SEC);
      draw_tv_ui_pcha(pch);
      tv_set_selection_by_rno(rno);
    }
}

void disp_channel_name_idx()
{
    disp_channel_name_pch(&chanlist[chan_idx]);
}

extern char output_filename[128];
void disp_rec_output_status();
void draw_current_time();

void disp_channel_name_pch_time(CHANLIST *pch)
{
  int rno=pch->rno;

  if (fftv_mode & FFTV_MODE_RADIO) {
    draw_radio_freq((float)pch->freq/1000);
    draw_radio_station_name(pch->name);
    radio_set_selection_by_name(pch->name);
  }
  else {
    char st[32];

    draw_current_time();
    sprintf(st,"%d", rno);
    draw_channel_name(st,CH_CLEAR_SEC);
    draw_tv_ui_pcha(pch);
//    tv_set_selection_by_rno(rno);
  }


  if (record_status==RECORD_ON)
    disp_rec_output_status();
}


void disp_channel_name_idx_time()
{
  disp_channel_name_pch_time(&chanlist[chan_idx]);
}


void free_chanlist()
{
  int i;

//  dbg("... free_chanlist\n");

  for(i=0;i<chanlistN;i++)
    free(chanlist[i].name);

  free(chanlist); chanlist=NULL;
  chanlistN=0;

  free(full_chan_arr);  full_chan_arr=NULL; full_chan_arrN=0;
}


void scan_channel();
extern char fftv_freq[128], ffradio_freq[128];

void load_chanlist()
{
  DIR *pdir;
  char *dirname=ffdir_name();
  char orig_dir[256];

  getcwd(orig_dir,sizeof(orig_dir));

  if (chdir(dirname))
    p_err("cannot change dir to %s\n",dirname);

  free_chanlist();

  if ((pdir=opendir("."))) {
    struct dirent *pdirent;
    int i;

    dbg("%s found, will use it\n",dirname);

    while ((pdirent=readdir(pdir))) {
      char chname[64], rno_fname[16], tv_schedule_html[256];
      int freq;
      int rno,flag=0;
      FILE *fp;

      if (pdirent->d_name[0]=='.')
        continue;
      if (!pdirent->d_name[0])
        continue;

      strcpy(rno_fname,pdirent->d_name);

      rno = atoi(rno_fname);

      if (rno<=0)
        continue;

      if ((fp=fopen(rno_fname,"r"))==NULL)
        continue;

      fscanf(fp,"%d %x\n",&freq, &flag);

      fgets(chname,sizeof(chname),fp);
      int chname_len=strlen(chname);
      if (chname[chname_len-1]=='\n')
        chname[chname_len-1]=0;

      tv_schedule_html[0]=0;
      fgets(tv_schedule_html,sizeof(tv_schedule_html),fp);
      int tv_schedule_html_len=strlen(tv_schedule_html);
      if (tv_schedule_html[tv_schedule_html_len-1]=='\n')
        tv_schedule_html[tv_schedule_html_len-1]=0;


      int ofs_x0=0, ofs_y0=0, ofs_x1=0, ofs_y1=0;
      fscanf(fp,"%d %d %d %d\n", &ofs_x0, &ofs_y0, &ofs_x1, &ofs_y1);

      fclose(fp);

      chanlist=realloc(chanlist,sizeof(CHANLIST)*(chanlistN+1));
      chanlist[chanlistN].name=strdup(chname);
      chanlist[chanlistN].freq=freq;
      chanlist[chanlistN].rno=rno;
      chanlist[chanlistN].flag=flag;
      chanlist[chanlistN].tv_schedule_html=strdup(tv_schedule_html);
      chanlist[chanlistN].ofs_x0 = ofs_x0;
      chanlist[chanlistN].ofs_y0 = ofs_y0;
      chanlist[chanlistN].ofs_x1 = ofs_x1;
      chanlist[chanlistN].ofs_y1 = ofs_y1;
      chanlistN++;
    }

    closedir(pdir);

    dbg("chanlistN:%d\n",chanlistN);

#if 1
    if (fftv_mode & FFTV_MODE_TV)
      sort_chanlist_rno();
    else
      sort_chanlist_freq();
#endif

    full_chan_arr=malloc(sizeof(int) * chanlistN);

    for(i=0;i<chanlistN;i++) {
      if (chanlist[i].flag & CHAN_IGNORE)
        continue;
      full_chan_arr[full_chan_arrN++]=chanlist[i].rno;
    }

  } else {
    p_err("cannot open %s\n",fftv_freq);
  }

  chdir(orig_dir);
}


extern int saved_tv_chan_idx;

void set_channel_pch_1(CHANLIST *pch)
{
   unsigned long freq=chfreq(pch->freq);

   pchan_current = pch;

   tv_audio_off();

   ioctl(vfd, VIDIOCSFREQ, &freq);

   usleep(100000);

   tv_audio_on();

   restore_channel_states();
}

char *load_channel_last_session();

void init_TV_SYS()
{
  struct stat sta;

  init_freq_dir();

  if (stat(fftv_freq,&sta)<0) {
    scan_channel();
  }
  else {
    load_chanlist();
  }


  char *channel_last_session=load_channel_last_session();

  dbg(".. tv channel_last_session %s\n",channel_last_session);

  if (channel_last_session)
    set_channel_by_name(channel_last_session);
  else
    set_channel_chan_idx(0);
}

extern int saved_fm_chan_idx;

void init_freq_dir();

void init_radio_SYS()
{
  int i;
  unsigned long freq;
  struct stat sta;

  dbg("init_radio_SYS\n");
  init_freq_dir();


  if (stat(ffradio_freq,&sta)<0) {
    radio_scan();
  }
  else {
    load_chanlist();
  }

  ioctl(vfd, VIDIOCGFREQ, &freq);

  for(i=0;i<chanlistN;i++) {
    unsigned long freqi=freq_khz_radiodev(chanlist[i].freq);

    if (iabs(freqi - freq) < 10) {
      set_channel_chan_idx(i);
      break;
    }
  }

  char *channel_last_session=load_channel_last_session();

  dbg("radio channel_last_session %s\n",channel_last_session);
  if (channel_last_session)
    set_channel_by_name(channel_last_session);
  else
    set_channel_chan_idx(0);

//  tv_audio_on();
}

static int chan_last_rno;

void set_channel_pch(CHANLIST *pch)
{
   unsigned long freq=chfreq(pch->freq);

   pchan_current = pch;

   if (xwin)
     disp_channel_name_pch(pch);

   tv_audio_off();

   ioctl(vfd, VIDIOCSFREQ, &freq);

   usleep(100000);

   tv_audio_on();

   if (xwin) {
     if (fftv_mode & FFTV_MODE_TV)
       change_win_name(pch->name);
   }

   restore_channel_states();
}


void set_channel_chan_idx()
{
   if (!chanlist)
     p_err("chanlist is null");

   if (chan_idx < 0 || chan_idx >= chanlistN)
     p_err("invalid idx %d",chan_idx);

   set_channel_pch(&chanlist[chan_idx]);
}


void set_chan_idx(int idx)
{
  pchan_current = &chanlist[idx];
  chan_idx=idx;
}


// currently for FM only
void set_tuner_freq(int freqkhz)
{
  int i;
  int minidx=-1;

  for(i=0;i<chanlistN;i++) {
    if (minidx <0 || iabs(chanlist[i].freq - freqkhz) <
        iabs(chanlist[minidx].freq - freqkhz)) {
      minidx = i;
    }
  }

  set_chan_idx(minidx);

  set_channel_chan_idx();
}


void set_channel_idx(int idx)
{
   unsigned long freq;

   if (idx >= chanlistN)
     idx=chanlistN;
   if (idx < 0)
     idx=0;

   freq=chfreq(chanlist[idx].freq);

   ioctl(vfd, VIDIOCSFREQ, &freq);
   restore_channel_states();
}


void set_channel_pch_0(CHANLIST *pch)
{
   unsigned long freq;

   freq=chfreq(pch->freq);

   ioctl(vfd, VIDIOCSFREQ, &freq);
}

int find_chanlist_name(char *chname)
{
  int i;

  for(i=0;i<chanlistN;i++)
    if (!strcmp(chanlist[i].name,chname))
      break;
  return i;
}

CHANLISTS *system_chanlist();

int setChannelFreq_0(char *chname)
{
  int i;

  i = find_chanlist_name(chname);

  if (i==chanlistN) {
    int rno=atoi(chname);

    for(i=0;i<chanlistN;i++)
      if (chanlist[i].rno == rno)
        break;

    if (i==chanlistN) {
      CHANLISTS *chlists;
      CHANLIST  *chlist=NULL;
      int chlistN=0;

      chlists=system_chanlist();
      chlist=chlists->list;
      chlistN=chlists->count;

      for(i=0;i<chlistN;i++) {
        if (!strcmp(chlist[i].name,chname) || chlist[i].rno==rno) {
          set_channel_pch(&chlist[i]);
          dbg("external ... channel\n");
          return 1;
        }
      }

      p_err("tv channel name '%s' is not defined in your tv system", chname);
    }
  }

  set_chan_idx(i);
  return 0;
}

void setChannelFreq(char *chname)
{
  if (setChannelFreq_0(chname))
    return;

  set_channel_chan_idx();
}


void set_channel_by_name(char *chname)
{
//  dbg("set_channel_by_name:%s\n",chname);
  setChannelFreq(chname);
}


void set_channel_by_rno(int rno)
{
  char tt[16];

  sprintf(tt,"%d\n",rno);
  setChannelFreq(tt);
}


#define STACK_N (16)

int channel_stack[STACK_N];
int channel_stackN;

void save_channel_last()
{
  int i;

//  dbg("mmm %d %d\n",chan_last_rno,chanlist[chan_idx].rno);

  if (chan_last_rno==chanlist[chan_idx].rno)
    return;

  chan_last_rno=chanlist[chan_idx].rno;

  for(i=0;i<channel_stackN;i++) {
    if (channel_stack[i]==chan_last_rno)
      break;
  }

  if (i < channel_stackN) {
    memcpy(&channel_stack[i],&channel_stack[i],sizeof(int)*channel_stackN-1-i);
    channel_stackN--;
  }

  if (channel_stackN)
    memmove(&channel_stack[1],channel_stack,sizeof(int)*(STACK_N - 1));

  channel_stack[0]=chan_last_rno;
  if (channel_stackN < STACK_N)
    channel_stackN++;
#if 0
  dbg("channel_stackN:%d\n",channel_stackN);
  for(i=0;i<channel_stackN;i++)
    dbg("  %d  %d\n",channel_stack[i], chanlist[channel_stack[i]].rno);
#endif
}

int rno2chidx(int rno)
{
  int i;

  for(i=0;i<chanlistN;i++)
    if (chanlist[i].rno==rno)
      break;

  return i;
}



int fftv_system_chanlists_idx()
{
  int i;
  char fftv_freq_system[256];
  char tvsys[256];
  FILE *fp;

  sprintf(fftv_freq_system,"%s/.system",fftv_freq);
  if ((fp=fopen(fftv_freq_system,"r"))==NULL)
    return -1;

  fscanf(fp,"%s",tvsys);

  for(i=0;i<chanlistsN;i++) {
    if (!strcmp(chanlists[i].name,tvsys)) {
      break;
    }
  }

  if (i==chanlistsN) {
    fprintf(stderr,"%s is not defined",tvsys);
  }

  return i;
}


CHANLISTS *system_chanlist()
{
  int i;

  i = fftv_system_chanlists_idx();

  if (i==chanlistsN)
    return NULL;

  return &chanlists[i];
}


CHANLIST *rno2pcha(int rno)
{
  int i;

  for(i=0;i<chanlistN;i++)
    if (chanlist[i].rno==rno) {
      return &chanlist[i];
    }

  if (fftv_mode & FFTV_MODE_RADIO)
    return NULL;

  CHANLISTS *chlists=system_chanlist();

  if (!chlists)
    return NULL;

  CHANLIST *chlist=chlists->list;
  int chlistN=chlists->count;

  for(i=0;i<chlistN;i++) {
    if (chlist[i].rno==rno) {
      return &chlist[i];
    }
  }

  return NULL;
}


void setChannelFreq_rno(int rno)
{
  int i;

  clear_remkeys();
  save_channel_last();
  i=rno2chidx(rno);

  if ((fftv_mode & FFTV_MODE_TV) && i==chanlistN) {
    CHANLISTS *chlists;
    CHANLIST  *chlist=NULL;
    int chlistN=0;

    chlists=system_chanlist();
    chlist=chlists->list;
    chlistN=chlists->count;

    for(i=0;i<chlistN;i++) {
      if (chlist[i].rno==rno) {
        set_channel_pch(&chlist[i]);

        return;
      }
    }

    return;
  }

  set_chan_idx(i);
  set_channel_chan_idx();
}


void prev_channel()
{
  clear_remkeys();
  save_channel_last();

  do {
    if (chan_idx == 0)
      set_chan_idx(chanlistN-1);
    else
      set_chan_idx(chan_idx-1);
  } while (pchan_current->flag & CHAN_IGNORE);

  set_channel_chan_idx();
}

void next_channel()
{
  clear_remkeys();
  save_channel_last();

  do {
    set_chan_idx((chan_idx+1)%chanlistN);
  } while (pchan_current->flag & CHAN_IGNORE);

  set_channel_chan_idx();
}


void return_last_channel()
{
//  dbg("%d %d\n",chan_last_rno, chanlist[chan_idx].rno);
  if (chan_last_rno && chan_last_rno!=chanlist[chan_idx].rno) {
    int t=chan_last_rno;

    chan_last_rno=chanlist[chan_idx].rno;
    set_chan_idx(rno2chidx(t));
    set_channel_chan_idx();
  }
}

#define FM_FINE_TUNE_STEP (25)
#define TV_FINE_TUNE_STEP (60)

void set_freq_disp(CHANLIST *pcha)
{
  int freq=chfreq(pcha->freq);

  ioctl(vfd, VIDIOCSFREQ, &freq);

  if (fftv_mode & FFTV_MODE_TV) {
    char tt[32];
    sprintf(tt,"Freq:%d",freq);
    draw_channel_name(tt,CH_CLEAR_SEC);
  }
}

int save_channel(CHANLIST *pcha)
{
  char *dirname=ffdir_name();
  char fname[256];
  FILE *fp;

  sprintf(fname,"%s/%d",dirname,pcha->rno);

  if ((fp=fopen(fname,"w"))==NULL) {
    perror(fname);
    return -1;
  }

  fprintf(fp,"%d %x\n%s\n%s\n",pcha->freq,pcha->flag,pcha->name,
          pcha->tv_schedule_html);
  fprintf(fp,"%d %d %d %d\n",pcha->ofs_x0,pcha->ofs_y0,pcha->ofs_x1,pcha->ofs_y1);
  fclose(fp);

  dbg("save_channel %d %d\n",pcha->rno,pcha->flag);

  return 0;
}

void freq_fine_tune_down()
{
  clear_remkeys();
  if (fftv_mode & FFTV_MODE_RADIO) {
    chanlist[chan_idx].freq-=FM_FINE_TUNE_STEP;
  } else {
    chanlist[chan_idx].freq-=TV_FINE_TUNE_STEP;
  }
  save_channel(&chanlist[chan_idx]);

  set_freq_disp(&chanlist[chan_idx]);

  if (fftv_mode & FFTV_MODE_RADIO)
    draw_channel_name_chidx();
}

void freq_fine_tune_up()
{
  clear_remkeys();
  if (fftv_mode & FFTV_MODE_RADIO) {
    chanlist[chan_idx].freq+=FM_FINE_TUNE_STEP;
  } else {
    chanlist[chan_idx].freq+=TV_FINE_TUNE_STEP;
  }

  save_channel(&chanlist[chan_idx]);

  set_freq_disp(&chanlist[chan_idx]);

  if (fftv_mode & FFTV_MODE_RADIO)
    draw_channel_name_chidx();
}

void delete_rno_file(int rno)
{
  char *dirname=ffdir_name();
  char fname[256];

  sprintf(fname,"%s/%d",dirname,rno);
  unlink(fname);
}

void delete_current_channel()
{
  int rno=pchan_current->rno;

  if (!pchan_current || !pchan_current->name) {
    p_err("null pchan_current");
    return;
  }

  if (fftv_mode & FFTV_MODE_TV) {
    pchan_current->flag = CHAN_IGNORE;
    save_channel(pchan_current);
  } else {
    delete_rno_file(rno);
  }
}

void add_current_channel()
{
  char *chname=pchan_current->name;
  int rno=pchan_current->rno;

  full_chan_arr=realloc(full_chan_arr, sizeof(int) * (chanlistN+1));
  full_chan_arr[chanlistN] = rno;

  dbg("add_current_channel ------ %s\n", chname);

  pchan_current->flag = 0;

  save_channel(pchan_current);
}

