#include <stdio.h>
#include <gtk/gtk.h>
#include "tv-freq.h"
#include <sys/types.h>
#include <linux/videodev.h>
#include "tv-audio.h"
#include "fftv.h"
#include <X11/Xlib.h>
#include <gdk/gdkx.h>

#if 0
#define FONT_FAMILY "charter"
#endif

GtkWidget *tv_ui_window;
Window x_tv_ui_window;
static GtkWidget *tv_ui_menubar;
static GtkWidget *button_freq, *button_name, *button_adj_dec, *button_adj_inc;
static GtkWidget *image_station_prev, *image_station_next;
static GtkWidget *event_box_station_prev, *event_box_station_next;
static GtkWidget *button_name_label, *button_freq_label;
static GtkWidget *hscale;
static GtkWidget *vbox_top, *hbox_volume,*hbox_station, *button_speaker;
static GtkWidget *vbox_station_left, *vbox_station_right, *hbox_freq_adj;
static GtkWidget *image_speaker;
static GtkWidget *hseparator1,*vseparator;
static GtkWidget *mts_label;

static GtkWidget *hseparator2;
static GtkWidget *hbox_record;
static GtkWidget *image_record, *image_pause, *image_stop;
static GtkWidget *event_box_record, *event_box_pause, *event_box_stop;


static GtkWidget *hseparator3;
static GtkWidget *hbox_monitor;
static GtkWidget *image_monitor_toggle, *image_monitor_swap, *image_fullscreen, *image_schedule;
static GtkWidget *button_monitor_toggle, *button_monitor_swap, *button_fullscreen, *button_schedule;

static GtkWidget *hseparator4;
static GtkWidget *hbox_pict_adj;
static GtkWidget *image_brightness, *vbox_brightness, *hbox_brightness,
                 *button_brightness_p, *button_brightness_m;
static GtkWidget *image_color, *vbox_color, *hbox_color,
                 *button_color_p, *button_color_m;
static GtkWidget *image_contrast, *vbox_contrast, *hbox_contrast,
                 *button_contrast_p, *button_contrast_m;
static GtkWidget *image_hue, *vbox_hue, *hbox_hue,
                 *button_hue_p, *button_hue_m;

static GtkWidget *vbox_mts, *hseparator_mts;
static GtkWidget *arrow_mts;


static void callback_button_station_prev( GtkWidget *widget,
                      gpointer   data )
{
    prev_channel();
}

static void callback_button_station_next( GtkWidget *widget, gpointer data )
{
    next_channel();
}


static void callback_button_enter_station_prev( GtkWidget *widget, gpointer data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_prev),ICON_BASENAME "/down40-bd.png");
}

static void callback_button_enter_station_next( GtkWidget *widget, gpointer data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_next),ICON_BASENAME "/up40-bd.png");
}


static void callback_button_leave_station_prev( GtkWidget *widget,
                      gpointer   data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_prev),ICON_BASENAME "/down40.png");
}

static void callback_button_leave_station_next( GtkWidget *widget,
                      gpointer   data )
{
    gtk_image_set_from_file(GTK_IMAGE(image_station_next),ICON_BASENAME "/up40.png");
}

void set_tv_record_active_image()
{
    gtk_image_set_from_file(GTK_IMAGE(image_record),ICON_BASENAME "/rec-ui0.png");
}

static void callback_button_enter_record( GtkWidget *widget, gpointer   data)
{
  set_tv_record_active_image();
}

void set_tv_record_inactive_image()
{
    if (image_record)
      gtk_image_set_from_file(GTK_IMAGE(image_record),ICON_BASENAME "/rec-ui1.png");
}

static void callback_button_leave_record( GtkWidget *widget, gpointer   data)
{
  if (record_status!=RECORD_ON)
    set_tv_record_inactive_image();
}


void start_recording();
static void callback_button_start_recording( GtkWidget *widget, gpointer   data)
{
    start_recording();
}

void set_tv_pause_active_image()
{
  if (image_pause)
    gtk_image_set_from_file(GTK_IMAGE(image_pause),ICON_BASENAME "/pause-ui0.png");
}

static void callback_button_enter_pause( GtkWidget *widget, gpointer   data)
{
    set_tv_pause_active_image();
}

void set_tv_pause_inactive_image()
{
  if (image_pause)
    gtk_image_set_from_file(GTK_IMAGE(image_pause),ICON_BASENAME "/pause-ui1.png");
}

static void callback_button_leave_pause(GtkWidget *widget, gpointer   data)
{
  if (record_status!=RECORD_PAUSE)
    set_tv_pause_inactive_image();
}

void pause_recording();
static void callback_button_pause_recording( GtkWidget *widget, gpointer   data )
{
    pause_recording();
}

static void callback_button_enter_stop( GtkWidget *widget, gpointer   data)
{
    gtk_image_set_from_file(GTK_IMAGE(image_stop),ICON_BASENAME "/stop-ui0.png");
}

static void callback_button_leave_stop( GtkWidget *widget, gpointer   data)
{
    gtk_image_set_from_file(GTK_IMAGE(image_stop),ICON_BASENAME "/stop-ui1.png");
}

void stop_recording();
static void callback_button_stop_recording( GtkWidget *widget, gpointer data)
{
    stop_recording();
}


void hide_tv_ui_window();

static gboolean delete_event( GtkWidget *widget,
                              GdkEvent  *event,
                              gpointer   data )
{
    hide_tv_ui_window();
    return TRUE;
}

static gboolean b_update_tv_hscale=FALSE;

void set_soundcard_volume(int val);

static void callback_hscale(GtkRange *range, gpointer user_data)
{
  int val=(int)gtk_range_get_value(range);

  if (!b_update_tv_hscale)
    set_soundcard_volume(val);
}

static void callback_button_adj_dec(GtkButton *button, gpointer user_data)
{
  freq_fine_tune_down();
}

static void callback_button_adj_inc(GtkButton *button, gpointer user_data)
{
  freq_fine_tune_up();
}

void toggle_channel_monitor();
static void callback_button_monitor_toggle(GtkButton *button, gpointer user_data)
{
  toggle_channel_monitor();
}

void swap_channel_monitor();
static void callback_button_monitor_swap(GtkButton *button, gpointer user_data)
{
  swap_channel_monitor();
}

void fullscreen_toggle640();

static void callback_button_fullscreen(GtkButton *button, gpointer user_data)
{
  fullscreen_toggle640();
}

void show_current_tv_schedule();

static void callback_button_schedule(GtkButton *button, gpointer user_data)
{
  show_current_tv_schedule();
}

void show_tv_list_window();
static void callback_button_freq(GtkButton *button, gpointer user_data)
{
  show_tv_list_window();
  printf("freq\n");
}

static void callback_button_name(GtkButton *button, gpointer user_data)
{
  show_tv_list_window();
  printf("name\n");
}

void soundcard_mute_toggle();
void update_tv_hscale();
static void callback_button_speaker(GtkButton *button, gpointer user_data)
{
  soundcard_mute_toggle();
  update_tv_hscale();
}

void set_pic_adj_idx(char *name);

void pic_val_dec(), pic_val_inc();
void pic_adj_select_menu();

static void callback_button_pict_adj_minus(GtkButton *button, gpointer user_data)
{
  pic_adj_select_menu();
  set_pic_adj_idx(user_data);
  pic_val_dec();
}

static void callback_button_pict_adj_plus(GtkButton *button, gpointer user_data)
{
  pic_adj_select_menu();
  set_pic_adj_idx(user_data);
  pic_val_inc();
}


static gboolean tv_ui_key_eve(GtkWidget *widget, GdkEventKey *event, gpointer user_data)
{
  gchar chr=0;
  if (event->string)
    chr=event->string[0];
//    feed_xkey(event->keyval,event->state,chr);
//  printf("key %x %x %c\n",event->keyval,event->state,chr);
  feed_xkey(event->keyval,event->state,chr);
  return TRUE;
}


void draw_tv_ui_pcha(CHANLIST *pch)
{
  if (!tv_ui_window)
    return;

//  gtk_window_set_title (GTK_WINDOW (tv_ui_window), pch->name);
  gtk_label_set_text(GTK_LABEL(button_name_label),pch->name);

  PangoFontDescription* font=pango_context_get_font_description
                             (gtk_widget_get_pango_context (button_name_label));

//  pango_font_description_set_family(font,FONT_FAMILY);
  pango_font_description_set_size(font,PANGO_SCALE * 20);
  gtk_widget_modify_font(button_name_label,font);

  GtkRequisition sz;
  gtk_widget_size_request(button_name_label, &sz);

  int scalex=2500 / (sz.width + 1) + 1;
  int scaley= 800 / (sz.height + 1) + 1;
  int scale=MIN(scalex,scaley);

  font=pango_context_get_font_description
             (gtk_widget_get_pango_context (button_name_label));

  pango_font_description_set_size(font,PANGO_SCALE*scale);
  gtk_widget_modify_font(button_name_label,font);

  char tt[8];
  sprintf(tt,"%d",pch->rno);
  gtk_label_set_text(GTK_LABEL(button_freq_label), tt);
}

void update_tv_hscale()
{
  int vol=get_soundcard_volume();

  if (!tv_ui_window)
    return;

  b_update_tv_hscale=TRUE;
  gtk_range_set_value(GTK_RANGE(hscale),vol);
  b_update_tv_hscale=FALSE;
}

extern int mts_mode_arrN, mts_mode_active_idx;
extern SOUND_MODE_M mts_mode_arr[];
void set_to_mts_mode_active_idx();

static void cb_update_menu_select(GtkWidget  *item,  gpointer data)
{
    /* Set the update policy for both scale widgets */
  int idx=GPOINTER_TO_INT(data);

  mts_mode_active_idx = idx;
  gtk_label_set_text(GTK_LABEL(mts_label),mts_mode_arr[idx].name);
  set_to_mts_mode_active_idx();
}

gint
my_popup_handler (GtkWidget *widget, GdkEvent *event)
{
  GtkMenu *menu;
  GdkEventButton *event_button;

  g_return_val_if_fail (widget != NULL, FALSE);
  g_return_val_if_fail (GTK_IS_MENU (widget), FALSE);
  g_return_val_if_fail (event != NULL, FALSE);

  menu = GTK_MENU (widget);

  if (event->type == GDK_BUTTON_PRESS) {
      event_button = (GdkEventButton *) event;
      gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
                      event_button->button, event_button->time);
      return TRUE;
  }

  return FALSE;
}

void switch_to_radio();
void create_about_window();
void create_codec_window();
void create_vid_src_window();
void create_schedule_browser_select_window();
void create_tv_dislay_option_window();
void create_prog_list_window_current_channel();
#if TCRON
void create_prog_list_window_current_channel();
void create_prog_list_window_view();
#else
void create_prog_list_window_current_channel()
{
   p_err("tcron 0.4.3 (November 2003 released) or above required to have reminder/recorder power-on capability");
}
void create_prog_list_window_view()
{
   p_err("tcron 0.4.3 (November 2003 released) or above required to have reminder/recorder power-on capability");
}
#endif

static GtkItemFactoryEntry tv_ui_menu_items[] = {
  { "/_Switch",         NULL,         NULL, 0, "<Branch>" },
  { "/_Switch/FM radio",   NULL,      switch_to_radio , 0, NULL },
  { "/_Switch/Video source",   NULL,  create_vid_src_window, 0, NULL },
  { "/_Switch/Reminder\\/Recorder current channel",   "<alt>R",  create_prog_list_window_current_channel, 0, NULL },
  { "/_Switch/Reminder\\/Recorder",   "<control>R",  create_prog_list_window_view, 0, NULL },
  { "/_Switch/sep1",     NULL,         NULL, 0, "<Separator>" },
  { "/_Switch/Quit",     "<control>Q", do_exit_fftv, 0, NULL },
  { "/_Config",         NULL,         NULL, 0, "<Branch>" },
  { "/_Config/File name\\/codec", NULL, create_codec_window, 0, NULL },
  { "/_Config/Display options", NULL, create_tv_dislay_option_window, 0, NULL },
  { "/_Config/TV Schedule brower", NULL,create_schedule_browser_select_window, 0, NULL },
  { "/_Help",         NULL,         NULL, 0, "<LastBranch>" },
  { "/_Help/About fftv", NULL, create_about_window, 0, NULL },
};

static void get_tv_ui_main_menu( GtkWidget  *window)
{
  GtkItemFactory *item_factory;
  GtkAccelGroup *accel_group;
  gint nmenu_items = sizeof (tv_ui_menu_items) / sizeof (tv_ui_menu_items[0]);

  accel_group = gtk_accel_group_new ();

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>",
				       accel_group);

  gtk_item_factory_create_items (item_factory, nmenu_items, tv_ui_menu_items, NULL);

  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  tv_ui_menubar = gtk_item_factory_get_widget (item_factory, "<main>");
}

void mouse_scroll_callback(GtkWidget *widget,GdkEvent *event, gpointer data);


void create_tv_ui_win()
{
    GdkColor prog_color;
    PangoFontDescription* font;

    /* Create a new tv_ui_window */
    tv_ui_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_wmclass(GTK_WINDOW(tv_ui_window),"fftv","fftv");
    GtkWidget *win_icon=gtk_image_new_from_file(ICON_BASENAME "/fftv.png");
    gtk_window_set_icon(GTK_WINDOW(tv_ui_window),gtk_image_get_pixbuf(GTK_IMAGE(win_icon)));

    /* Set the tv_ui_window title */
    gtk_window_set_title (GTK_WINDOW (tv_ui_window), "fftv control panel");

    /* Set a handler for delete_event that immediately
     * exits GTK. */
    g_signal_connect (G_OBJECT (tv_ui_window), "delete_event",
                      G_CALLBACK (delete_event), NULL);

    g_signal_connect(G_OBJECT(tv_ui_window),"key-press-event", G_CALLBACK(tv_ui_key_eve), NULL);

    gtk_container_set_border_width (GTK_CONTAINER (tv_ui_window), 6);

    vbox_top = gtk_vbox_new (FALSE, 0);

    get_tv_ui_main_menu(tv_ui_window);

    gtk_box_pack_start (GTK_BOX (vbox_top), tv_ui_menubar, FALSE, FALSE, 0);

    GtkWidget *hseparator0=gtk_hseparator_new();

    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator0, FALSE, FALSE, 2);

    /* Put the table in the main tv_ui_window */
    gtk_container_add (GTK_CONTAINER (tv_ui_window), vbox_top);

    /* Create first button */
    button_freq = gtk_button_new();
    button_freq_label = gtk_label_new("    ");
    font=pango_context_get_font_description
                               (gtk_widget_get_pango_context (button_freq_label));
#ifdef FONT_FAMILY
    pango_font_description_set_family(font,FONT_FAMILY);
#endif
    pango_font_description_set_size(font,PANGO_SCALE*30);

    prog_color.pixel=prog_color.red=prog_color.green=prog_color.blue=0;
    prog_color.blue=65535;

    gtk_widget_modify_font(button_freq_label,font);
    gtk_widget_modify_fg(button_freq_label,GTK_STATE_NORMAL,&prog_color);
    gtk_widget_modify_fg(button_freq_label,GTK_STATE_PRELIGHT,&prog_color);
    gtk_container_add (GTK_CONTAINER (button_freq), button_freq_label);

    g_signal_connect (G_OBJECT (button_freq),"clicked",
      G_CALLBACK(callback_button_freq),NULL);


    button_name = gtk_button_new();
    button_name_label = gtk_label_new("");
    font=pango_context_get_font_description
                               (gtk_widget_get_pango_context (button_name_label));
#ifdef FONT_FAMILY
    pango_font_description_set_family(font,FONT_FAMILY);
#endif
    pango_font_description_set_size(font,PANGO_SCALE*10);

    prog_color.pixel=0;
    prog_color.red=255<<8;
    prog_color.green=255<<8;
    prog_color.blue=0<<8;

    gtk_widget_modify_font(button_name_label,font);
#if 0
    gtk_widget_modify_fg(button_name_label,GTK_STATE_NORMAL,&prog_color);
    gtk_widget_modify_fg(button_name_label,GTK_STATE_PRELIGHT,&prog_color);
#endif
    gtk_container_add (GTK_CONTAINER (button_name), button_name_label);

    g_signal_connect (G_OBJECT (button_name),"clicked",
      G_CALLBACK(callback_button_name),NULL);


    vbox_station_left = gtk_vbox_new (TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_station_left), button_freq, TRUE, TRUE, 2);
    gtk_box_pack_start (GTK_BOX (vbox_station_left), button_name, TRUE, TRUE, 2);


    hscale = gtk_hscale_new_with_range (0,100,2);
    g_signal_connect (G_OBJECT (hscale), "value-changed",
                      G_CALLBACK (callback_hscale), NULL);

    button_speaker = gtk_button_new();
    image_speaker = gtk_image_new_from_file(ICON_BASENAME "/gnome-mixer-small.png");
    gtk_container_add (GTK_CONTAINER (button_speaker), image_speaker);

    GtkTooltips *button_speaker_tips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_speaker_tips), button_speaker, "Mute",NULL);


    g_signal_connect (G_OBJECT (button_speaker),"clicked",
      G_CALLBACK(callback_button_speaker),NULL);


    vbox_mts = gtk_vbox_new (FALSE, 0);

    GtkWidget *menu = gtk_menu_new ();

    int i;

    for(i=0;i<mts_mode_arrN;i++) {
      GtkWidget *item = gtk_menu_item_new_with_label (mts_mode_arr[i].name);

      gtk_widget_show(item);

      g_signal_connect (G_OBJECT (item), "activate",
                        G_CALLBACK (cb_update_menu_select), GINT_TO_POINTER(i));

      gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    }


    GtkWidget *mts_button = gtk_button_new();
    mts_label = gtk_label_new(mts_mode_arr[mts_mode_active_idx].name);
    gtk_box_pack_start(GTK_BOX (vbox_mts), mts_label, TRUE,TRUE,0);
    hseparator_mts = gtk_hseparator_new();
    gtk_box_pack_start(GTK_BOX (vbox_mts), hseparator_mts, TRUE,TRUE,0);
    g_signal_connect_swapped (GTK_OBJECT (mts_button), "button_press_event",
          G_CALLBACK (my_popup_handler), GTK_OBJECT (menu));

    arrow_mts = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_OUT);
    gtk_box_pack_start(GTK_BOX (vbox_mts),arrow_mts,TRUE,TRUE,0);

    gtk_container_add (GTK_CONTAINER (mts_button), vbox_mts);

    hbox_volume = gtk_hbox_new (FALSE, 0);

    gtk_box_pack_start (GTK_BOX (hbox_volume), button_speaker, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_volume), mts_button, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (hbox_volume), hscale, TRUE, TRUE, 0);


    button_adj_dec = gtk_button_new_with_label ("-");
    button_adj_inc = gtk_button_new_with_label ("+");
    hbox_freq_adj = gtk_hbox_new (TRUE, 0);

    GtkTooltips *button_adj_dec_tips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_adj_dec_tips), button_adj_dec, "freq decr",NULL);
    GtkTooltips *button_adj_inc_tips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_adj_inc_tips), button_adj_inc, "freq incr",NULL);

    gtk_box_pack_start(GTK_BOX (hbox_freq_adj),button_adj_dec,TRUE,TRUE,0);
    gtk_box_pack_start(GTK_BOX (hbox_freq_adj),button_adj_inc,TRUE,TRUE,0);
    g_signal_connect (G_OBJECT (button_adj_dec),"clicked",
       G_CALLBACK(callback_button_adj_dec),NULL);
    g_signal_connect (G_OBJECT (button_adj_inc),"clicked",
       G_CALLBACK(callback_button_adj_inc),NULL);


    image_station_prev = gtk_image_new_from_file(ICON_BASENAME "/down40.png");
    image_station_next = gtk_image_new_from_file (ICON_BASENAME "/up40.png");
    event_box_station_prev = gtk_event_box_new();
    event_box_station_next = gtk_event_box_new();

    gtk_container_add (GTK_CONTAINER (event_box_station_prev), image_station_prev);
    gtk_container_add (GTK_CONTAINER (event_box_station_next), image_station_next);
    vbox_station_right = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox_station_right),event_box_station_next, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox_station_right),event_box_station_prev, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX(vbox_station_right),hbox_freq_adj, TRUE, TRUE, 2);


    g_signal_connect (G_OBJECT (event_box_station_prev), "button_press_event",
                      G_CALLBACK (callback_button_station_prev), NULL);
    g_signal_connect (G_OBJECT (event_box_station_next), "button_press_event",
                      G_CALLBACK (callback_button_station_next), NULL);

    g_signal_connect (G_OBJECT (event_box_station_prev), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_station_prev), NULL);
    g_signal_connect (G_OBJECT (event_box_station_next), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_station_next), NULL);

    g_signal_connect (G_OBJECT (event_box_station_prev), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_station_prev), NULL);
    g_signal_connect (G_OBJECT (event_box_station_next), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_station_next), NULL);

    hbox_station = gtk_hbox_new (FALSE, 0);
    vseparator=gtk_vseparator_new();
    gtk_box_pack_start(GTK_BOX(hbox_station),vbox_station_left,TRUE,TRUE, 0);
    gtk_box_pack_start (GTK_BOX(hbox_station),vseparator, FALSE, FALSE, 2);
    gtk_box_pack_start(GTK_BOX(hbox_station),vbox_station_right,FALSE,FALSE, 0);

    hseparator1=gtk_hseparator_new();

    hseparator2=gtk_hseparator_new();
    hbox_record = gtk_hbox_new (TRUE, 0);

    image_record = gtk_image_new_from_file(ICON_BASENAME "/rec-ui1.png");
    image_pause = gtk_image_new_from_file (ICON_BASENAME "/pause-ui1.png");
    image_stop = gtk_image_new_from_file (ICON_BASENAME "/stop-ui1.png");
    event_box_record = gtk_event_box_new();
    event_box_pause = gtk_event_box_new();
    event_box_stop = gtk_event_box_new();

    g_signal_connect (G_OBJECT (event_box_record), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_record), NULL);
    g_signal_connect (G_OBJECT (event_box_record), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_record), NULL);
    g_signal_connect (G_OBJECT (event_box_record), "button_press_event",
                      G_CALLBACK (callback_button_start_recording), NULL);

    g_signal_connect (G_OBJECT (event_box_pause), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_pause), NULL);
    g_signal_connect (G_OBJECT (event_box_pause), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_pause), NULL);
    g_signal_connect (G_OBJECT (event_box_pause), "button_press_event",
                      G_CALLBACK (callback_button_pause_recording), NULL);

    g_signal_connect (G_OBJECT (event_box_stop), "enter-notify-event",
                      G_CALLBACK (callback_button_enter_stop), NULL);
    g_signal_connect (G_OBJECT (event_box_stop), "leave-notify-event",
                      G_CALLBACK (callback_button_leave_stop), NULL);
    g_signal_connect (G_OBJECT (event_box_stop), "button_press_event",
                      G_CALLBACK (callback_button_stop_recording), NULL);

    gtk_container_add (GTK_CONTAINER (event_box_record), image_record);
    gtk_container_add (GTK_CONTAINER (event_box_pause), image_pause);
    gtk_container_add (GTK_CONTAINER (event_box_stop), image_stop);

    gtk_box_pack_start(GTK_BOX(hbox_record),event_box_record,TRUE,TRUE, 0);
    gtk_box_pack_start (GTK_BOX(hbox_record),event_box_pause, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_record),event_box_stop,FALSE,FALSE, 0);

    hseparator3=gtk_hseparator_new();

    hbox_monitor = gtk_hbox_new (TRUE, 0);

    image_monitor_toggle = gtk_image_new_from_file(ICON_BASENAME "/monitor_toggle.png");
    image_monitor_swap = gtk_image_new_from_file(ICON_BASENAME "/monitor_swap.png");
    image_fullscreen = gtk_image_new_from_file(ICON_BASENAME "/fullscreen.png");
    image_schedule = gtk_image_new_from_file(ICON_BASENAME "/schedule.png");

    button_monitor_toggle = gtk_button_new();
    button_monitor_swap = gtk_button_new();
    button_fullscreen = gtk_button_new();
    button_schedule = gtk_button_new();

    g_signal_connect (G_OBJECT (button_monitor_toggle),"clicked",
       G_CALLBACK(callback_button_monitor_toggle),NULL);
    g_signal_connect (G_OBJECT (button_monitor_swap),"clicked",
       G_CALLBACK(callback_button_monitor_swap),NULL);
    g_signal_connect (G_OBJECT (button_fullscreen),"clicked",
       G_CALLBACK(callback_button_fullscreen),NULL);
    g_signal_connect (G_OBJECT (button_schedule),"clicked",
       G_CALLBACK(callback_button_schedule),NULL);

    gtk_container_add (GTK_CONTAINER (button_monitor_toggle), image_monitor_toggle);
    gtk_container_add (GTK_CONTAINER (button_monitor_swap), image_monitor_swap);
    gtk_container_add (GTK_CONTAINER (button_fullscreen), image_fullscreen);
    gtk_container_add (GTK_CONTAINER (button_schedule), image_schedule);

    GtkTooltips *button_monitor_toggle_tips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_monitor_toggle_tips), button_monitor_toggle, "Channel Monitor Toggle On/Off",NULL);

    GtkTooltips *button_monitor_swap_tips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_monitor_swap_tips), button_monitor_swap, "Channel Monitor/Main swap",NULL);

    GtkTooltips *button_fullscreen_tips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_fullscreen_tips), button_fullscreen, "Fullscreen",NULL);

    GtkTooltips *button_schedule_tips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_schedule_tips), button_schedule, "Invoke schedule html browser",NULL);

    gtk_box_pack_start(GTK_BOX(hbox_monitor),button_monitor_toggle,FALSE,FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_monitor),button_monitor_swap, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_monitor),button_fullscreen, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_monitor),button_schedule, FALSE, FALSE, 0);

    hseparator4=gtk_hseparator_new();

    hbox_pict_adj = gtk_hbox_new (TRUE, 0);

    vbox_brightness = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_pict_adj),vbox_brightness,FALSE,FALSE, 0);
    image_brightness = gtk_image_new_from_file(ICON_BASENAME "/brightness.png");
    gtk_box_pack_start(GTK_BOX(vbox_brightness),image_brightness,FALSE,FALSE, 0);
    hbox_brightness = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_brightness),hbox_brightness,FALSE,FALSE, 0);
    button_brightness_m = gtk_button_new_with_label("-");
    button_brightness_p = gtk_button_new_with_label("+");
    gtk_box_pack_start(GTK_BOX(hbox_brightness),button_brightness_m,TRUE,TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_brightness),button_brightness_p,TRUE,TRUE, 0);
    g_signal_connect (G_OBJECT (button_brightness_m),"clicked",
       G_CALLBACK(callback_button_pict_adj_minus), "brightness");
    g_signal_connect (G_OBJECT (button_brightness_p),"clicked",
       G_CALLBACK(callback_button_pict_adj_plus), "brightness");


    vbox_contrast = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_pict_adj),vbox_contrast,FALSE,FALSE, 2);
    image_contrast = gtk_image_new_from_file(ICON_BASENAME "/contrast.png");
    gtk_box_pack_start(GTK_BOX(vbox_contrast),image_contrast,FALSE,FALSE, 0);
    hbox_contrast = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_contrast),hbox_contrast,FALSE,FALSE, 0);
    button_contrast_m = gtk_button_new_with_label("-");
    button_contrast_p = gtk_button_new_with_label("+");
    gtk_box_pack_start(GTK_BOX(hbox_contrast),button_contrast_m,TRUE,TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_contrast),button_contrast_p,TRUE,TRUE, 0);
    g_signal_connect (G_OBJECT (button_contrast_m),"clicked",
       G_CALLBACK(callback_button_pict_adj_minus), "contrast");
    g_signal_connect (G_OBJECT (button_contrast_p),"clicked",
       G_CALLBACK(callback_button_pict_adj_plus), "contrast");


    vbox_color = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_pict_adj),vbox_color,FALSE,FALSE, 2);
    image_color = gtk_image_new_from_file(ICON_BASENAME "/color.png");
    gtk_box_pack_start(GTK_BOX(vbox_color),image_color,FALSE,FALSE, 0);
    hbox_color = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_color),hbox_color,FALSE,FALSE, 0);
    button_color_m = gtk_button_new_with_label("-");
    button_color_p = gtk_button_new_with_label("+");
    gtk_box_pack_start(GTK_BOX(hbox_color),button_color_m,TRUE,TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_color),button_color_p,TRUE,TRUE, 0);
    g_signal_connect (G_OBJECT (button_color_m),"clicked",
       G_CALLBACK(callback_button_pict_adj_minus), "color");
    g_signal_connect (G_OBJECT (button_color_p),"clicked",
       G_CALLBACK(callback_button_pict_adj_plus), "color");


    vbox_hue = gtk_vbox_new (FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_pict_adj),vbox_hue,FALSE,FALSE, 2);
    image_hue = gtk_image_new_from_file(ICON_BASENAME "/hue.png");
    gtk_box_pack_start(GTK_BOX(vbox_hue),image_hue,FALSE,FALSE, 0);
    hbox_hue = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start(GTK_BOX(vbox_hue),hbox_hue,FALSE,FALSE, 0);
    button_hue_m = gtk_button_new_with_label("-");
    button_hue_p = gtk_button_new_with_label("+");
    gtk_box_pack_start(GTK_BOX(hbox_hue),button_hue_m,TRUE,TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox_hue),button_hue_p,TRUE,TRUE, 0);
    g_signal_connect (G_OBJECT (button_hue_m),"clicked",
       G_CALLBACK(callback_button_pict_adj_minus), "hue");
    g_signal_connect (G_OBJECT (button_hue_p),"clicked",
       G_CALLBACK(callback_button_pict_adj_plus), "hue");


    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_station, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator1, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_volume, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator2, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_record, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator3, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_monitor, FALSE, FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox_top), hseparator4, FALSE, FALSE, 2);
    gtk_box_pack_start (GTK_BOX (vbox_top), hbox_pict_adj, FALSE, FALSE, 0);

    g_signal_connect (G_OBJECT(tv_ui_window), "scroll-event",
                      G_CALLBACK (mouse_scroll_callback), NULL);

    draw_tv_ui_pcha(pchan_current);

    gtk_widget_show_all (tv_ui_window);

    dbg("tv win created ..\n");
}

extern GtkWidget *gtv_win;

void show_tv_ui_window()
{
  if (!tv_ui_window)
    create_tv_ui_win();
  else {
    gtk_window_present(GTK_WINDOW(tv_ui_window));
  }

  x_tv_ui_window = GDK_WINDOW_XWINDOW(tv_ui_window->window);

  if (!gtv_win)
    create_tv_win();

  update_tv_hscale();
}

void hide_tv_ui_window()
{
  if (tv_ui_window)
    gtk_widget_hide(tv_ui_window);
}
