// tv win grab mode display

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <linux/videodev.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <time.h>
#include <sys/shm.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/extensions/XShm.h>
#include <X11/extensions/Xvlib.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <string.h>
#include <ctype.h>
#include <errno.h>

#include "tv-freq.h"
#include "fftv.h"
#include "vidstr.h"
#include "tv-win.h"

extern Window root;
extern GC gc;

static struct video_mmap gb_buf;
static guchar *video_buf;
static struct video_mbuf gb_buffers;
static int tv_in_image_size;
static int grab_sizeX, grab_sizeY;
static int grab_sizeX_cur, grab_sizeY_cur;
static int grab_pixN;
static int gb_frame;

static int shm_error_handler(Display * dpy, XErrorEvent * event)
{
    p_err("X11 SHM error");
    return 0;
}

static XvPortID         xvport;

extern Display *dpy;
static int screen_depth_bytes=3;

static Visual *xvisual;
static int xdepth;
static XImage *ximage;
static XShmSegmentInfo xshminfo;

static XvImage *xvimage;
static int xvmode;


extern Window tv_win;
extern GC gc;
void init_Xv_port();

void set_grab_frame_size(int sizeX, int sizeY)
{
  grab_sizeX = sizeX;
  grab_sizeY = sizeY;
  grab_pixN = grab_sizeX * grab_sizeY;
}

void init_xvisual()
{
  if (tv_win_disp_mode & (TV_GRAB_XV|TV_GRAB_XV_RECORD)) {
    init_Xv_port();
  }
  else {
    GdkVisual* gvisual = gdk_visual_get_best_with_type (GDK_VISUAL_TRUE_COLOR);

    xvisual = GDK_VISUAL_XVISUAL(gvisual);
    xdepth = gvisual->depth;
    screen_depth_bytes = xdepth / 8;
    dbg("xvisual:%x  %d\n", xvisual, xdepth);

  }
}

void close_image()
{
  XShmDetach(dpy, &xshminfo);

  if (xvimage) {
    XFree(xvimage);
    xvimage = NULL;
  } else
  if (ximage) {
    XDestroyImage(ximage);
    ximage = NULL;
  }

  shmdt(xshminfo.shmaddr);
}


void create_x_shm_image()
{
//  void            *orig_handler;
  int data_size;

  if (!xvport)
    init_Xv_port();

  if (grab_sizeX_cur == grab_sizeX && grab_sizeY_cur == grab_sizeY)
    return;

  grab_sizeX_cur = grab_sizeX; grab_sizeY_cur = grab_sizeY;

  if (xvimage || ximage)
    close_image();

  bzero(&xshminfo, sizeof(XShmSegmentInfo));

  if (tv_win_disp_mode == TV_GRAB_MIT_SHM) {
    ximage = XShmCreateImage(dpy, xvisual, xdepth, ZPixmap, NULL, &xshminfo,
                             grab_sizeX, grab_sizeY);
    if (!ximage)
         p_err("X11 SHM error 2");

    data_size = ximage->bytes_per_line * grab_sizeY;
  }
  else {
    dbg("XvShmCreateImage: %d %d\n", grab_sizeX, grab_sizeY);
    if (!(xvimage = XvShmCreateImage(dpy, xvport, xvmode, NULL,
                                     grab_sizeX, grab_sizeY, &xshminfo)))
         p_err("X11 SHM error 2");

    data_size = xvimage->data_size;
  }

  dbg("shmget\n");
  xshminfo.shmid = shmget(IPC_PRIVATE, data_size, IPC_CREAT | 0777);

  if (xshminfo.shmid < 0) {
       p_err("X11 SHM error 3  %d", xshminfo.shmid);
  }

  xshminfo.shmaddr = (char *) shmat(xshminfo.shmid, 0, 0);
  if ((void *)-1 == xshminfo.shmaddr) {
       p_err("X11 SHM error 4");
  }

  if (tv_win_disp_mode == TV_GRAB_MIT_SHM)
    ximage->data = xshminfo.shmaddr;
  else
    xvimage->data = xshminfo.shmaddr;

  xshminfo.readOnly = False;

  dbg("XShmAttach\n");
  XShmAttach(dpy, &xshminfo);
  XSync(dpy, False);
  shmctl(xshminfo.shmid, IPC_RMID, 0);
}



static void put_x_shm_image(int x, int y)
{
  XShmPutImage(dpy,tv_win, gc, ximage, 0,0, x,y, win_xl, win_yl, False);
}

void put_xv_image(char *image)
{

  memcpy(xvimage->data, image, grab_pixN + grab_pixN/2);

  if (!disp_rectN) {
//    dbg("put_xv_image full\n");
    XvPutImage(dpy, xvport, tv_win, gc, xvimage,
                 0, 0, grab_sizeX, grab_sizeY, /* source */
                 0, 0, win_xl, win_yl /* dest */);
    return;
  }

//  dbg("disp_rectN:%d\n", disp_rectN);

  gdk_flush();

  int i;
  for(i=0; i< disp_rectN; i++) {
    RECT *prect = &disp_rect[i];
    int imgx0 = prect->x0 * grab_sizeX / win_xl;
    int imgx1 = prect->x1 * grab_sizeX / win_xl;

    int imgy0 = prect->y0 * grab_sizeY / win_yl;
    int imgy1 = prect->y1 * grab_sizeY / win_yl;
    int imgsizeX = imgx1 - imgx0 + 1;
    int imgsizeY = imgy1 - imgy0 + 1;

//    dbg("put_xv_image %d,%d\n", imgx0,imgy0, imgsizeX, imgsizeY);

    XvPutImage(dpy, xvport, tv_win, gc, xvimage,
                 imgx0, imgy0, imgsizeX, imgsizeY,
                 prect->x0, prect->y0,
                 prect->x1 - prect->x0 + 1,
                 prect->y1 - prect->y0 + 1);

  }
}


#include <sys/mman.h>

void fftv_grab_close()
{
  if (!video_buf)
    return;
  stop_refresh_grab_tv_win();
  munmap(video_buf, gb_buffers.size);
  video_buf = NULL;
}

void init_v4l_mbuf()
{
   fftv_grab_close();

   int ret = ioctl(vfd,VIDIOCGMBUF, &gb_buffers);

   dbg("frames:%d\n", gb_buffers.frames);

   if (ret < 0) {
     p_err("VIDIOCGMBUF error %d\n",ret);
   }

   video_buf = mmap(0, gb_buffers.size, PROT_READ|PROT_WRITE,MAP_SHARED, vfd, 0);
   if ((guchar*)-1 == video_buf) {
       p_err("mmap failed");
   }

   struct video_picture vpict;
   bzero(&vpict,sizeof(vpict));
   ioctl(vfd,VIDIOCGPICT,&vpict);

   tv_in_image_size = grab_sizeX * grab_sizeY * (vpict.depth / 8);

   gb_buf.height = grab_sizeY;
   gb_buf.width = grab_sizeX;
   gb_buf.format = VIDEO_PALETTE_YUV420P;
   gb_buf.frame = 0;

   grab_pixN = grab_sizeX * grab_sizeY;
#if 0
   ret = ioctl(vfd, VIDIOCMCAPTURE, &gb_buf);

   if (ret < 0) {
     perror("VIDIOCMCAPTURE");
     p_err("%d\n", ret);
   }
#endif
   dbg("init_v4l_mbuf %d  depth:%d  pal:%d\n", tv_in_image_size, vpict.depth, vpict.palette);
}

gchar *fftv_read_frame()
{
  gb_buf.frame = (gb_frame + 1) % gb_buffers.frames;

  if (ioctl(vfd, VIDIOCMCAPTURE, &gb_buf) < 0) {
      if (errno == EAGAIN)
          fprintf(stderr,"Cannot Sync\n");
      else
          perror("fftv_read_frame:VIDIOCMCAPTURE");
      return FALSE;
  }

  while (ioctl(vfd, VIDIOCSYNC, &gb_frame) < 0 &&
         (errno == EAGAIN || errno == EINTR));

  gchar *imageptr = video_buf + gb_buffers.offsets[gb_frame];

  gb_frame = gb_buf.frame;
  return imageptr;
}


gboolean refresh_grab_tv_win()
{
#if 1
  if (!tv_win)
    return TRUE;

  guchar *imageptr = fftv_read_frame();

  if (tv_win_disp_mode == TV_GRAB_MIT_SHM) {
    memcpy(ximage->data, imageptr, tv_in_image_size);
    put_x_shm_image(0,0);
  }
  else
    put_xv_image(imageptr);

#endif
  return TRUE;
}

static int local_win_x, local_win_y, local_win_sizeX, local_win_sizeY;

void grab_draw_local_win(int x, int y, int sizeX, int sizeY)
{
  guchar *imageptr = fftv_read_frame();

  local_win_x = x;  local_win_y = y;
  local_win_sizeX = sizeX;   local_win_sizeY = sizeY;

//  dbg("grab_draw_local_win: %d %d %d %d\n", x,y, sizeX,  sizeY);

  memcpy(xvimage->data, imageptr, grab_pixN + grab_pixN/2);

  XvPutImage(dpy, xvport, tv_win, gc, xvimage,
                 0, 0, grab_sizeX, grab_sizeY, /* source */
                 x, y, sizeX, sizeY /* dest */);
}


gboolean refresh_local_win()
{
  if (!tv_win)
    return TRUE;
#if 0
  dbg("refresh_local_win: %d %d %d %d\n",
    local_win_x, local_win_y, local_win_sizeX, local_win_sizeY);
#endif
  grab_draw_local_win(local_win_x, local_win_y, local_win_sizeX, local_win_sizeY);
  return TRUE;
}




void init_Xv_port()
{
  XvAdaptorInfo *adaptorArr, *padaptor;
  int adaptorArrN, i, j, k;

  if (XvQueryAdaptors(dpy, root, &adaptorArrN,&adaptorArr) != Success)
    p_err("cannot XvQueryAdaptors error");

//  dbg("adaptorArrN: %d\n", adaptorArrN);

  for (i=0; i<adaptorArrN; i++) {
    padaptor = &adaptorArr[i];

    if (!((padaptor->type & XvInputMask) && (padaptor->type & XvImageMask)) )
      continue;

//    dbg("padaptor->num_ports: %d\n", padaptor->num_ports);


    for (j=0; j<padaptor->num_ports;j++) {
      XvImageFormatValues *imgFormatArr=NULL;
      int imgFormatArrN=0;

      xvport = padaptor->base_id + j;
      imgFormatArr = XvListImageFormats(dpy, xvport, &imgFormatArrN);

//      dbg("imgFormatArrN:%d\n", imgFormatArrN);

      if (!imgFormatArr || !imgFormatArrN)
        continue;

      if (XvGrabPort(dpy, xvport, CurrentTime) != Success)
        continue;


      for (k=0; k< imgFormatArrN; k++) {
        char *tt=(char *)&imgFormatArr[k].id;

//        dbg("zzzzzzzzzzzz %c%c%c%c\n",tt[0],tt[1],tt[2],tt[3]);

        xvmode = imgFormatArr[k].id;
        if (!memcmp(&xvmode,"I420",4)) {

          goto xvport_found;
        }

      }

      XvUngrabPort(dpy, xvport, CurrentTime);
    }
  }

  p_err("YUV420P(I420) format is not found in your XVideo interface");

xvport_found:
  XvFreeAdaptorInfo(adaptorArr);

  dbg("xvport:%x\n", xvport);
}
