#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>

#include <linux/videodev.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>

#include <gtk/gtk.h>
#include <gdk/gdkx.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xresource.h>
#include <X11/Xproto.h>
#include <X11/extensions/XTest.h>
#include <X11/keysym.h>
#include <X11/extensions/xf86vmode.h>


#include "reminder.h"

#include <string.h>
#include <ctype.h>

#include "tv-freq.h"
#include "fftv.h"
#include "xkey.h"
#include "vidstr.h"
#include "tv-win.h"

int Row=4,Col=4;
int ch_per_page=16;
int page_chan_idx=0;
int pre_cx, pre_cy;
int pre_vxl, pre_vyl;
static int *preview_arr;
static int preview_arrN;
extern int *full_chan_arr, full_chan_arrN;
extern int timeout_tag_refresh_grab_tv_win;
extern int tv_win_frame_rate_normal;
gboolean refresh_local_win();
extern Display *dpy;
extern GC gc_sfont;
extern long foreground, background;
extern Window tv_win;
extern int sfwidth,sfheight;
extern GtkWidget *gtv_win;
extern GdkWindow *gdk_tv_win;
extern GdkGC *g_gc;
extern int full_win;
extern int full_win640;

void change_win_name(char *str);
void getTVWinXY(int *gx, int *gy);
void set_channel_idx(int idx);
void disp_channel_name_idx();
extern int tv_win_mouse_x, tv_win_mouse_y;
void draw_tv_preview();

static int tag_preview_move_timeout;
static gboolean preview_move_timeout()
{
  draw_tv_preview();

  tag_preview_move_timeout=0;

  return FALSE;
}


void add_tv_preview_timer(int msec)
{
  if (tag_preview_move_timeout)
    gtk_timeout_remove(tag_preview_move_timeout);

  tag_preview_move_timeout=gtk_timeout_add(msec,preview_move_timeout,NULL);
}


void set_local_win_frame_rate()
{
  stop_refresh_grab_tv_win();

  timeout_tag_refresh_grab_tv_win =
    gtk_timeout_add((1000+2) / tv_win_frame_rate_normal,
       refresh_local_win, NULL);
}

static void pre_cxy_idx()
{
  int x,y;
  int preidx=page_chan_idx + pre_cy * Col + pre_cx;

  x=pre_cx*pre_vxl;
  y=(Row-1-pre_cy)*pre_vyl;
  if (preidx<0)
    preidx=0;
  if (preidx >= preview_arrN)
    preidx %= preview_arrN;
  int rno=preview_arr[preidx];

  set_channel_idx(rno2chidx(rno));

  if (TV_WIN_DISP_MODE_GRAB) {
    usleep(110000);
    local_vwin(x,y,pre_vxl,pre_vyl);
  } else {
    local_vwin(x,y,pre_vxl,pre_vyl);
    usleep(110000);
  }

  tv_audio_on();

  if (TV_WIN_DISP_MODE_GRAB)
    set_local_win_frame_rate();
}

static void draw_pre_tv_ch_rno(int rno, int x, int y)
{
   char s[16];
   int i,j;

   sprintf(s,"%d",rno);

   XSetForeground(dpy,gc_sfont,background);

#define SHADOW (2)

   for(i=0;i<SHADOW;i++)
     for(j=0;j<SHADOW;j++)
       XDrawString(dpy,tv_win,gc_sfont,x+i,y+sfheight+j,s,strlen(s));

   XSetForeground(dpy,gc_sfont,foreground);
   XDrawString(dpy,tv_win,gc_sfont,x,y+sfheight,s,strlen(s));
   XFlush(dpy);
}

void update_pre_win_size()
{
  pre_vxl=win_xl/Col; pre_vyl=win_yl/Row;
}


#if 0
static GdkImage *tv_win_image;
static int tv_win_image_xl, tv_win_image_yl;
static void save_tv_win_image()
{
  if (tv_win_image)
    g_object_unref(tv_win_image);
  tv_win_image = gdk_drawable_get_image(gtv_win->window, 0, 0, win_xl, win_yl);
  tv_win_image_xl = win_xl;
  tv_win_image_yl = win_yl;
}

static void restore_tv_win_image()
{
  gdk_draw_image(gtv_win->window, g_gc, tv_win_image, 0, 0, 0, 0, win_xl, win_yl);
}
#endif

void draw_tv_preview()
{
  int ix,iy;
  int pidx;


  dbg("draw_tv_preview....\n");


  if (TV_WIN_DISP_MODE_GRAB)
    stop_refresh_grab_tv_win();

  if (preview_arr==full_chan_arr)
    change_win_name("Channels Preview ");
  else
    change_win_name("Channels Selection History");

  update_pre_win_size();
  tv_audio_off();

  pidx=page_chan_idx;

  for(iy=Row-1;iy>=0;iy--) {
    int y=iy*pre_vyl;

    for(ix=0;ix<Col;ix++) {
      int x=ix*pre_vxl;
      int x0=x+win_x,y0=y+win_y;
      int x1=x0+pre_vxl-1,y1=y0+pre_vyl-1;
      int i;


      for(i=0;i<clipsN;i++) {
        int cx0=win_x+clips[i].x,cy0=win_y+clips[i].y;
        int cx1=cx0 + clips[i].width -1;
        int cy1=cy0 + clips[i].height -1;

        if (x0>=cx0 && y0>=cy0 && x1<=cx1 && y1<=cy1)
          break;
      }

      if (i<clipsN) {
//        dbg("skip %d %d\n",ix,iy);
        continue;
      }

      if (pidx>=preview_arrN)
        pidx = preview_arrN - 1;

      int rno=preview_arr[pidx];

      set_channel_pch_0(rno2pcha(rno));

      if (TV_WIN_DISP_MODE_GRAB) {

        fftv_read_frame();
        usleep(34000);
        local_vwin(x,y,pre_vxl,pre_vyl);
      } else {
        local_vwin(x,y,pre_vxl,pre_vyl);
        usleep(110000);
        tv_pic_off();
      }

      if (!TV_WIN_DISP_MODE_GRAB)
        getTVWinXY(&win_x,&win_y);

      draw_pre_tv_ch_rno(rno,x,y);

      pidx=(pidx+1)%chanlistN;
    }
  }

  pre_cxy_idx();
}


static void before_preview_switch()
{
  int x,y,preidx,rno;

  x=pre_cx*pre_vxl; y=(Row-1-pre_cy)*pre_vyl;
  preidx=page_chan_idx + pre_cy * Col + pre_cx;
  if (preidx >= preview_arrN)
    preidx = preview_arrN - 1;
  rno=preview_arr[preidx];

  tv_audio_off();

  usleep(110000);

  if (!TV_WIN_DISP_MODE_GRAB)
    tv_pic_off();

  draw_pre_tv_ch_rno(rno,x,y);
}

void fullscreen_toggle();

void preview_select_channel()
{
   int preidx;

   set_fftv_cmd_state("Normal");
   preidx=page_chan_idx + pre_cy * Col + pre_cx;
   if (preidx >= preview_arrN)
     preidx = preview_arrN - 1;

   set_chan_idx(rno2chidx(preview_arr[preidx]));
   set_channel_chan_idx();
   set_vwin();
   tv_audio_on();
   tv_preview_mode=0;
   if (full_win)
     fullscreen_toggle();

   change_win_name(pchan_current->name);
   disp_channel_name_idx();
}



void preview_cursor_left()
{
  before_preview_switch();

  pre_cx--;
  if (pre_cx<0)
    pre_cx=Col-1;
  pre_cxy_idx();
}

void preview_cursor_right()
{
  before_preview_switch();

  pre_cx=(pre_cx+1)%Row;
  pre_cxy_idx();
}

void preview_cursor_up()
{
  before_preview_switch();
  pre_cy=(pre_cy+1)%Row;
  pre_cxy_idx();
}

void preview_cursor_down()
{
  before_preview_switch();
  pre_cy--;
  if (pre_cy<0)
    pre_cy=Row-1;
  pre_cxy_idx();
}


void PreviewSelMouseXY()
{
  int tpre_cx=tv_win_mouse_x / pre_vxl;
  int tpre_cy=Row - 1 - (tv_win_mouse_y / pre_vyl);

  if (tpre_cx==pre_cx && tpre_cy==pre_cy) {
    preview_select_channel();
    return;
  }

  before_preview_switch();

  pre_cx=tpre_cx;
  pre_cy=tpre_cy;
  pre_cxy_idx();
}



void hide_tv_list_window();
void hide_tv_ui_window();
void save_channel_last();

void preview_page_up()
{
  dbg("preview_page_up\n");

  if (!(fftv_mode & FFTV_MODE_TV))
     return;

  hide_tv_list_window();
  hide_tv_ui_window();


  set_fftv_cmd_state("Preview");
  if (!tv_preview_mode) {
    tv_preview_mode=1;
    save_channel_last();
    preview_arr=full_chan_arr;
    preview_arrN=full_chan_arrN;
  }
  else
    page_chan_idx+=ch_per_page;

  if (page_chan_idx>=preview_arrN)
    page_chan_idx=0;


  add_tv_preview_timer(300);
  gtk_main_iteration_do(False);
}

void preview_page_down()
{
   if (!(fftv_mode & FFTV_MODE_TV))
     return;

  hide_tv_list_window();
  hide_tv_ui_window();

   set_fftv_cmd_state("Preview");
   if (!tv_preview_mode) {
     tv_preview_mode=1;
     save_channel_last();
     preview_arr=full_chan_arr;
     preview_arrN=full_chan_arrN;
   }
   else
     page_chan_idx-=ch_per_page;

   if (page_chan_idx<0)
     page_chan_idx=preview_arrN-ch_per_page;

  add_tv_preview_timer(300);
  gtk_main_iteration_do(False);
}

extern int channel_stack[], channel_stackN;
void preview_channel_stack()
{
  preview_arr=channel_stack;
  preview_arrN=channel_stackN;
  pre_cx=pre_cy=0;
  draw_tv_preview();
}
